@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2007-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2007-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -24,6 +24,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <glib.h>
+#include <glib-compatible/glib-compatible.h>
 #include <gmodule.h>
 
 #ifdef HAVE_LIBBFD
@@ -61,6 +62,7 @@ struct _SymbolNames
     gchar *attributes_setup_function_name;
     gboolean require_data_setup_function;
     gboolean cpp;
+    gboolean gcc;
 };
 
 typedef struct _CutLoaderPrivate	CutLoaderPrivate;
@@ -94,7 +96,8 @@ symbol_names_new (gchar *namespace,
                   gchar *data_setup_function_name,
                   gchar *attributes_setup_function_name,
                   gboolean require_data_setup_function,
-                  gboolean cpp)
+                  gboolean cpp,
+                  gboolean gcc)
 {
     SymbolNames *names;
 
@@ -106,6 +109,7 @@ symbol_names_new (gchar *namespace,
     names->attributes_setup_function_name = attributes_setup_function_name;
     names->require_data_setup_function = require_data_setup_function;
     names->cpp = cpp;
+    names->gcc = gcc;
 
     return names;
 }
@@ -320,7 +324,7 @@ cut_loader_set_base_directory (CutLoader *loader, const gchar *base_directory)
 }
 
 static inline const gchar *
-skip_cpp_namespace (const gchar *name, GString *namespaces)
+skip_cpp_namespace_gcc (const gchar *name, GString *namespaces)
 {
     if (g_str_has_prefix(name, "_Z")) {
         name += strlen("_Z");
@@ -354,12 +358,12 @@ skip_cpp_namespace (const gchar *name, GString *namespaces)
 }
 
 static inline gboolean
-find_cpp_test_name (const gchar *name, gchar **test_name_start,
-                    guint64 *test_name_length, GString *namespaces)
+find_cpp_test_name_gcc (const gchar *name, gchar **test_name_start,
+                        guint64 *test_name_length, GString *namespaces)
 {
     guint64 length;
 
-    name = skip_cpp_namespace(name, namespaces);
+    name = skip_cpp_namespace_gcc(name, namespaces);
     if (!name)
         return FALSE;
 
@@ -374,7 +378,7 @@ find_cpp_test_name (const gchar *name, gchar **test_name_start,
 }
 
 static inline SymbolNames *
-detect_cpp_test_function_symbol_names (const gchar *name)
+detect_cpp_test_function_symbol_names_gcc (const gchar *name)
 {
     const gchar *original_name = name;
     gchar *test_name_start;
@@ -386,8 +390,8 @@ detect_cpp_test_function_symbol_names (const gchar *name)
     GString *test_name;
 
     test_name = g_string_new(NULL);
-    if (!find_cpp_test_name(name, &test_name_start, &test_name_length,
-                            test_name)) {
+    if (!find_cpp_test_name_gcc(name, &test_name_start, &test_name_length,
+                                test_name)) {
         g_string_free(test_name, TRUE);
         return NULL;
     }
@@ -451,9 +455,163 @@ detect_cpp_test_function_symbol_names (const gchar *name)
                             data_setup_function_name,
                             attributes_setup_function_name,
                             require_data_setup_function,
+                            TRUE,
                             TRUE);
 }
 
+static inline gboolean
+is_gcc_cpp_symbol (const gchar *name)
+{
+    return g_str_has_prefix(name, "_Z");
+}
+
+/* FIXME: Symbol detection codes for C++ are too dirty.
+          We should clean it up as soon as possible!!! */
+/*
+  format:
+    '?FUNCTION_NAME@NAMESPACE_N@NAMESPACE_N-1@...@NAMESPACE_0@@SIGNATURE'
+  SIGNATURE_NAME:
+    'YAXXZ': void XXX(void);
+    'YAXPAX@Z': void XXX(void *);
+    'YAXPBX@Z': void XXX(const void *);
+*/
+static inline SymbolNames *
+detect_cpp_test_function_symbol_names_vcc (const gchar *name)
+{
+    const gchar *original_name = name;
+    const gchar *test_name_start;
+    guint test_name_length;
+    gchar *data_setup_function_name = NULL;
+    gchar *attributes_setup_function_name = NULL;
+    gboolean require_data_setup_function = FALSE;
+    GString *test_name;
+
+    test_name_start = name + strlen("?");
+    if (!g_str_has_prefix(test_name_start, TEST_NAME_PREFIX))
+        return NULL;
+    test_name_length = 0;
+    while (test_name_start[test_name_length] &&
+           test_name_start[test_name_length] != '@') {
+        test_name_length++;
+    }
+    if (test_name_start[test_name_length] != '@')
+        return NULL;
+
+    test_name = g_string_new(NULL);
+    g_string_append_len(test_name, test_name_start, test_name_length);
+
+    name = test_name_start + test_name_length;
+    while (name[0]) {
+        const gchar *namespace;
+        guint namespace_length = 0;
+
+        if (name[0] != '@') {
+            g_string_free(test_name, TRUE);
+            return NULL;
+        }
+        namespace = name + strlen("@");
+
+        if (namespace[0] == '@') {
+            name = namespace + 1;
+            break;
+        }
+
+        while (namespace[namespace_length] &&
+               namespace[namespace_length] != '@') {
+            namespace_length++;
+        }
+        g_string_prepend(test_name, "::");
+        g_string_prepend_len(test_name, namespace, namespace_length);
+
+        name = namespace + namespace_length;
+    }
+
+    if (g_str_equal(name, "YAXXZ")) {
+    } else if (g_str_equal(name, "YAXPAX@Z") || g_str_equal(name, "YAXPBX@Z")) {
+        GString *data_setup_function_name_string;
+        size_t test_name_prefix_length;
+
+        require_data_setup_function = TRUE;
+
+        test_name_prefix_length = strlen(TEST_NAME_PREFIX);
+
+        data_setup_function_name_string = g_string_new(NULL);
+        g_string_append_len(data_setup_function_name_string,
+                            original_name, original_name - test_name_start);
+        g_string_append(data_setup_function_name_string,
+                        DATA_SETUP_FUNCTION_NAME_PREFIX);
+        g_string_append_len(data_setup_function_name_string,
+                            test_name_start + test_name_prefix_length,
+                            name - test_name_start + test_name_prefix_length);
+        g_string_append(data_setup_function_name_string, "YAXXZ");
+
+        data_setup_function_name = g_string_free(data_setup_function_name_string,
+                                                 FALSE);
+    } else {
+        g_string_free(test_name, TRUE);
+        return NULL;
+    }
+
+    {
+        GString *attributes_setup_function_name_string;
+        size_t test_name_prefix_length;
+
+        test_name_prefix_length = strlen(TEST_NAME_PREFIX);
+
+        attributes_setup_function_name_string = g_string_new(NULL);
+        g_string_append_len(attributes_setup_function_name_string,
+                            original_name, original_name - test_name_start);
+        g_string_append(attributes_setup_function_name_string,
+                        ATTRIBUTES_SETUP_FUNCTION_NAME_PREFIX);
+        g_string_append_len(attributes_setup_function_name_string,
+                            test_name_start + test_name_prefix_length,
+                            name - test_name_start + test_name_prefix_length);
+        g_string_append(attributes_setup_function_name_string, "YAXXZ");
+
+        attributes_setup_function_name =
+            g_string_free(attributes_setup_function_name_string, FALSE);
+    }
+
+    {
+        gchar *last_separator;
+        gchar *test_case_name;
+
+        last_separator = g_strrstr(test_name->str, "::");
+        if (last_separator) {
+            test_case_name = g_strndup(test_name->str,
+                                       last_separator - test_name->str);
+        } else {
+            test_case_name = g_strdup("");
+        }
+        return symbol_names_new(test_case_name,
+                                g_string_free(test_name, FALSE),
+                                g_strdup(original_name),
+                                data_setup_function_name,
+                                attributes_setup_function_name,
+                                require_data_setup_function,
+                                TRUE,
+                                FALSE);
+    }
+}
+
+static inline gboolean
+is_vcc_cpp_symbol (const gchar *name)
+{
+    return g_str_has_prefix(name, "?");
+}
+
+static inline SymbolNames *
+detect_cpp_test_function_symbol_names (const gchar *name)
+{
+    if (is_gcc_cpp_symbol(name)) {
+        return detect_cpp_test_function_symbol_names_gcc(name);
+    } else if (is_vcc_cpp_symbol(name)) {
+        return detect_cpp_test_function_symbol_names_vcc(name);
+    }
+
+    return NULL;
+}
+
 static inline SymbolNames *
 detect_test_function_symbol_names (const gchar *name)
 {
@@ -478,6 +636,7 @@ detect_test_function_symbol_names (const gchar *name)
                                 data_setup_function_name,
                                 attributes_setup_function_name,
                                 FALSE,
+                                FALSE,
                                 FALSE);
     }
 
@@ -718,15 +877,15 @@ is_including_test_name (const gchar *function_name, SymbolNames *names)
         guint64 function_name_length;
 
         target_namespace = g_string_new(NULL);
-        target_namespace_end = skip_cpp_namespace(function_name,
-                                                  target_namespace);
+        target_namespace_end = skip_cpp_namespace_gcc(function_name,
+                                                      target_namespace);
         if (!target_namespace_end) {
             g_string_free(target_namespace, TRUE);
             return FALSE;
         }
 
         test_namespace = g_string_new(NULL);
-        skip_cpp_namespace(names->test_function_name, test_namespace);
+        skip_cpp_namespace_gcc(names->test_function_name, test_namespace);
         included = g_str_equal(target_namespace->str, test_namespace->str);
         g_string_free(target_namespace, TRUE);
         g_string_free(test_namespace, TRUE);
@@ -760,7 +919,7 @@ is_valid_attribute_function_name (const gchar *function_name, SymbolNames *names
         return FALSE;
 
     if (names->cpp) {
-        if (!find_cpp_test_name(function_name, &base_name, NULL, NULL))
+        if (!find_cpp_test_name_gcc(function_name, &base_name, NULL, NULL))
             return FALSE;
     } else {
         base_name = (gchar *)function_name;
@@ -783,7 +942,7 @@ get_attribute_name (const gchar *attribute_function_name, SymbolNames *names)
     if (names->cpp) {
         const gchar *namespace_end;
 
-        namespace_end = skip_cpp_namespace(attribute_function_name, NULL);
+        namespace_end = skip_cpp_namespace_gcc(attribute_function_name, NULL);
         g_ascii_strtoull(namespace_end, &base_name, 10);
     } else {
         base_name = (gchar *)attribute_function_name;
@@ -890,7 +1049,7 @@ register_valid_test (CutLoader *loader, CutTestCase *test_case,
 }
 
 static const gchar *
-mangle (GString *buffer, const gchar *namespace, const gchar *function)
+mangle_gcc (GString *buffer, const gchar *namespace, const gchar *function)
 {
     gchar **components, **component;
 
@@ -911,6 +1070,43 @@ mangle (GString *buffer, const gchar *namespace, const gchar *function)
     return buffer->str;
 }
 
+static const gchar *
+mangle_vcc (GString *buffer, const gchar *namespace, const gchar *function)
+{
+    gchar **components, **component;
+
+    if (!namespace)
+        return function;
+
+    g_string_assign(buffer, "@YAXXZ");
+    components = g_strsplit(namespace, "::", 0);
+    for (component = components; *component; component++) {
+        g_string_prepend(buffer, "@");
+        g_string_prepend(buffer, *component);
+    }
+    g_strfreev(components);
+
+    g_string_prepend(buffer, "@");
+    g_string_prepend(buffer, function);
+    g_string_prepend(buffer, "?");
+
+    return buffer->str;
+}
+
+static const gchar *
+mangle (GString *buffer, const gchar *namespace, const gchar *function,
+        gboolean gcc)
+{
+    if (!namespace)
+        return function;
+
+    if (gcc) {
+        return mangle_gcc(buffer, namespace, function);
+    } else {
+        return mangle_vcc(buffer, namespace, function);
+    }
+}
+
 static void
 cb_complete (CutTestCase *test_case, CutTestContext *test_context,
              gboolean success, gpointer data)
@@ -920,7 +1116,7 @@ cb_complete (CutTestCase *test_case, CutTestContext *test_context,
 }
 
 static CutTestCase *
-create_test_case (CutLoader *loader, const gchar *namespace)
+create_test_case (CutLoader *loader, const gchar *namespace, gboolean gcc)
 {
     CutLoaderPrivate *priv;
     CutTestCase *test_case;
@@ -928,14 +1124,14 @@ create_test_case (CutLoader *loader, const gchar *namespace)
     CutStartupFunction startup = NULL;
     CutShutdownFunction shutdown = NULL;
     CutTeardownFunction teardown = NULL;
-    gchar *test_case_name, *filename;
+    gchar *test_case_name;
     GString *buffer;
 
     priv = CUT_LOADER_GET_PRIVATE(loader);
 
     buffer = g_string_new(NULL);
 #define MANGLE(function_name)                   \
-    mangle(buffer, namespace, function_name)
+    mangle(buffer, namespace, function_name, gcc)
 #define GET_HOOK_FUNCTION(name)                                         \
     if (!g_module_symbol(priv->module, MANGLE("cut_" #name), (gpointer)&name)) \
         g_module_symbol(priv->module, MANGLE(#name), (gpointer)&name)
@@ -953,16 +1149,23 @@ create_test_case (CutLoader *loader, const gchar *namespace)
 #undef MANGLE
     g_string_free(buffer, TRUE);
 
-    filename = g_path_get_basename(priv->so_filename);
-    if (g_str_has_prefix(filename, "lib")) {
-        gchar *string;
-        string = g_strdup(filename + strlen("lib"));
+    if (namespace) {
+        test_case_name = g_strdup(namespace);
+    } else {
+        gchar *filename;
+
+        filename = g_path_get_basename(priv->so_filename);
+        if (g_str_has_prefix(filename, "lib")) {
+            gchar *string;
+            string = g_strdup(filename + strlen("lib"));
+            g_free(filename);
+            filename = string;
+        }
+        test_case_name =
+            g_strndup(filename,
+                      strlen(filename) - strlen(G_MODULE_SUFFIX) -1);
         g_free(filename);
-        filename = string;
     }
-    test_case_name = g_strndup(filename,
-                               strlen(filename) - strlen(G_MODULE_SUFFIX) -1);
-    g_free(filename);
     test_case = cut_test_case_new(test_case_name,
                                   setup, teardown,
                                   startup, shutdown);
@@ -976,50 +1179,20 @@ create_test_case (CutLoader *loader, const gchar *namespace)
     return test_case;
 }
 
-typedef struct _LoadTestCaseData
+static gint
+find_cpp_test_case (gconstpointer value1, gconstpointer value2)
 {
-    CutLoader *loader;
-    GList *test_cases;
-} LoadTestCaseData;
-
-static gboolean
-load_test_case (gpointer key, gpointer value, gpointer user_data)
-{
-    const gchar *namespace = key;
-    GList *test_names = value;
-    LoadTestCaseData *data = user_data;
-    GList *node;
     CutTestCase *test_case;
+    SymbolNames *names;
 
-    test_case = create_test_case(data->loader, namespace);
-    for (node = test_names; node; node = g_list_next(node)) {
-        SymbolNames *names = node->data;
-        register_valid_test(data->loader, test_case, names);
-    }
-    g_list_free(test_names);
-    data->test_cases = g_list_append(data->test_cases, test_case);
-
-    return TRUE;
-}
-
-static guint
-string_hash_safe (gconstpointer value)
-{
-    if (!value)
+    test_case = CUT_TEST_CASE(value1);
+    names = (SymbolNames *)value2;
+    if (g_str_equal(cut_test_get_name(CUT_TEST(test_case)),
+                    names->namespace)) {
         return 0;
-
-    return g_str_hash(value);
-}
-
-static gboolean
-string_equal_safe (gconstpointer value1, gconstpointer value2)
-{
-    const gchar *string1 = value1;
-    const gchar *string2 = value2;
-
-    if (string1 == string2)
-        return TRUE;
-    return g_str_equal(string1, string2);
+    } else {
+        return 1;
+    }
 }
 
 GList *
@@ -1027,9 +1200,8 @@ cut_loader_load_test_cases (CutLoader *loader)
 {
     CutLoaderPrivate *priv;
     GList *node;
-    GList *test_names;
-    GHashTable *grouped_test_names;
-    LoadTestCaseData data;
+    GList *test_names, *test_cases = NULL;
+    CutTestCase *c_test_case = NULL;
 
     priv = CUT_LOADER_GET_PRIVATE(loader);
     if (!priv->so_filename)
@@ -1069,26 +1241,34 @@ cut_loader_load_test_cases (CutLoader *loader)
     if (!test_names)
         return NULL;
 
-    grouped_test_names = g_hash_table_new(string_hash_safe, string_equal_safe);
     for (node = test_names; node; node = g_list_next(node)) {
         SymbolNames *names = node->data;
-        GList *test_name_list;
+        CutTestCase *test_case = NULL;
 
-        test_name_list = g_hash_table_lookup(grouped_test_names,
-                                             names->namespace);
-        test_name_list = g_list_append(test_name_list, names);
-        g_hash_table_insert(grouped_test_names, names->namespace,
-                            test_name_list);
+        if (names->namespace) {
+            GList *test_case_node;
+            test_case_node = g_list_find_custom(test_cases, names,
+                                                find_cpp_test_case);
+            if (test_case_node)
+                test_case = test_case_node->data;
+            if (!test_case) {
+                test_case = create_test_case(loader, names->namespace,
+                                             names->gcc);
+                test_cases = g_list_prepend(test_cases, test_case);
+            }
+        } else {
+            if (!c_test_case) {
+                c_test_case = create_test_case(loader, NULL, FALSE);
+                test_cases = g_list_prepend(test_cases, c_test_case);
+            }
+            test_case = c_test_case;
+        }
+        register_valid_test(loader, test_case, names);
+        symbol_names_free(names);
     }
-    data.loader = loader;
-    data.test_cases = NULL;
-    g_hash_table_foreach_remove(grouped_test_names, load_test_case, &data);
-    g_hash_table_destroy(grouped_test_names);
-
-    g_list_foreach(test_names, (GFunc)symbol_names_free, NULL);
     g_list_free(test_names);
 
-    return data.test_cases;
+    return test_cases;
 }
 
 CutTestCase *
