@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2007-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2007-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -38,6 +38,7 @@ char **environ = NULL;
 
 #include "cut-listener.h"
 #include "cut-analyzer.h"
+#include "cut-file-stream-reader.h"
 #include "cut-test-runner.h"
 #include "cut-test-suite.h"
 #include "cut-ui.h"
@@ -47,9 +48,14 @@ char **environ = NULL;
 #include "../gcutter/gcut-main.h"
 #include "../gcutter/gcut-error.h"
 
+#ifdef G_OS_WIN32
+#  include <winsock2.h>
+#endif
+
 typedef enum {
     MODE_TEST,
-    MODE_ANALYZE
+    MODE_ANALYZE,
+    MODE_PLAY
 } RunMode;
 
 static gboolean environment_initialized = FALSE;
@@ -57,6 +63,7 @@ static gboolean initialized = FALSE;
 static RunMode mode = MODE_TEST;
 static gchar *test_directory = NULL;
 static gchar *log_directory = NULL;
+static gchar *log_file = NULL;
 static gchar *source_directory = NULL;
 static gchar **test_case_names = NULL;
 static gchar **test_names = NULL;
@@ -92,6 +99,8 @@ parse_mode (const gchar *option_name, const gchar *value,
         mode = MODE_TEST;
     } else if (g_utf8_collate(value, "analyze") == 0) {
         mode = MODE_ANALYZE;
+    } else if (g_utf8_collate(value, "play") == 0) {
+        mode = MODE_PLAY;
     } else {
         g_set_error(error,
                     G_OPTION_ERROR,
@@ -129,7 +138,7 @@ static const GOptionEntry option_entries[] =
     {"version", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, print_version,
      N_("Show version"), NULL},
     {"mode", 0, 0, G_OPTION_ARG_CALLBACK, parse_mode,
-     N_("Set run mode (default: test)"), "[test|analyze]"},
+     N_("Set run mode (default: test)"), "[test|analyze|play]"},
     {"source-directory", 's', 0, G_OPTION_ARG_STRING, &source_directory,
      N_("Set directory of source code"), "DIRECTORY"},
     {"name", 'n', 0, G_OPTION_ARG_STRING_ARRAY, &test_names,
@@ -224,7 +233,9 @@ cut_init (int *argc, char ***argv)
     program_name = g_path_get_basename((*argv)[0]);
     parameter_string =
         g_strdup_printf(N_("TEST_DIRECTORY\n"
-                           "  %s --mode=analyze %s LOG_DIRECTORY"),
+                           "  %s --mode=analyze %s LOG_DIRECTORY\n"
+                           "  %s --mode=play %s LOG_FILE"),
+                        program_name, _("[OPTION...]"),
                         program_name, _("[OPTION...]"));
     option_context = g_option_context_new(parameter_string);
     g_free(program_name);
@@ -267,23 +278,48 @@ cut_init (int *argc, char ***argv)
     }
 
     if (*argc == 1) {
+#if GLIB_CHECK_VERSION(2, 14, 0)
         gchar *help_string;
         help_string = g_option_context_get_help(option_context, TRUE, NULL);
         g_print("%s", help_string);
         g_free(help_string);
         g_option_context_free(option_context);
+#else
+        gint help_argc = 2;
+        gchar *help_argv[2];
+        gchar **help_argv_p;
+        help_argv[0] = (*argv)[0];
+        help_argv[1] = "--help";
+        help_argv_p = help_argv;
+        g_option_context_parse(option_context, &help_argc, &help_argv_p, NULL);
+        g_option_context_free(option_context);
+#endif
         exit(EXIT_FAILURE);
     }
 
-    if (mode == MODE_ANALYZE)
-        log_directory = (*argv)[1];
-    else
+    switch (mode) {
+    case MODE_TEST:
         test_directory = (*argv)[1];
+        break;
+    case MODE_ANALYZE:
+        log_directory = (*argv)[1];
+        break;
+    case MODE_PLAY:
+        log_file = (*argv)[1];
+        break;
+    }
 
 #ifdef HAVE_LIBBFD
     bfd_init();
 #endif
 
+#ifdef G_OS_WIN32
+    {
+        WSADATA wsa_data;
+        WSAStartup(MAKEWORD(2, 2), &wsa_data);
+    }
+#endif
+
     cut_ui_init();
 
     g_option_context_free(option_context);
@@ -332,16 +368,16 @@ cut_quit (void)
 
     cut_quit_environment();
 
+#ifdef G_OS_WIN32
+    WSACleanup();
+#endif
+
     initialized = FALSE;
 }
 
-CutRunContext *
-cut_create_run_context (void)
+void
+cut_setup_run_context (CutRunContext *run_context)
 {
-    CutRunContext *run_context;
-
-    run_context = cut_test_runner_new();
-
     cut_run_context_set_test_directory(run_context, test_directory);
     cut_run_context_set_log_directory(run_context, log_directory);
     if (source_directory)
@@ -364,7 +400,15 @@ cut_create_run_context (void)
                                                                 enable_convenience_attribute_definition);
     cut_run_context_set_stop_before_test(run_context, stop_before_test);
     cut_run_context_set_command_line_args(run_context, original_argv);
+}
 
+CutRunContext *
+cut_create_run_context (void)
+{
+    CutRunContext *run_context;
+
+    run_context = cut_test_runner_new();
+    cut_setup_run_context(run_context);
     return run_context;
 }
 
@@ -436,10 +480,15 @@ cut_start_run_context (CutRunContext *run_context)
     listeners = create_listeners();
     add_listeners(run_context, listeners);
 
-    if (mode == MODE_ANALYZE)
-        ui = NULL;
-    else
+    switch (mode) {
+    case MODE_TEST:
+    case MODE_PLAY:
         ui = get_cut_ui(listeners);
+        break;
+    case MODE_ANALYZE:
+        ui = NULL;
+        break;
+    }
 
     if (ui)
         success = cut_ui_run(ui, run_context);
@@ -452,26 +501,66 @@ cut_start_run_context (CutRunContext *run_context)
     return success;
 }
 
+static gboolean
+cut_run_in_test_mode (void)
+{
+    CutRunContext *run_context;
+    gboolean success;
+
+    run_context = cut_create_run_context();
+    success = cut_start_run_context(run_context);
+    g_object_unref(run_context);
+
+    return success;
+}
+
+static gboolean
+cut_run_in_analyze_mode (void)
+{
+    CutAnalyzer *analyzer;
+    GError *error = NULL;
+    gboolean success;
+
+    analyzer = cut_analyzer_new();
+    success = cut_analyzer_analyze(analyzer, log_directory, &error);
+    if (error) {
+        cut_utils_report_error(error);
+    }
+    g_object_unref(analyzer);
+
+    return success;
+}
+
+
+static gboolean
+cut_run_in_play_mode (void)
+{
+    CutRunContext *run_context;
+    gboolean success;
+
+    run_context = cut_file_stream_reader_new(log_file);
+    cut_setup_run_context(run_context);
+    success = cut_start_run_context(run_context);
+    g_object_unref(run_context);
+
+    return success;
+}
+
 gboolean
 cut_run (void)
 {
-    CutRunContext *run_context;
-    CutAnalyzer *analyzer;
     gboolean success = TRUE;
 
-    if (mode == MODE_ANALYZE) {
-        GError *error = NULL;
-
-        analyzer = cut_analyzer_new();
-        success = cut_analyzer_analyze(analyzer, log_directory, &error);
-        if (error) {
-            cut_utils_report_error(error);
-        }
-        g_object_unref(analyzer);
-    } else {
-        run_context = cut_create_run_context();
-        success = cut_start_run_context(run_context);
-        g_object_unref(run_context);
+    switch (mode) {
+    case MODE_TEST:
+        success = cut_run_in_test_mode();
+        break;
+    case MODE_ANALYZE:
+        success = cut_run_in_analyze_mode();
+        break;
+    case MODE_PLAY:
+        success = cut_run_in_play_mode();
+        break;
     }
 
     return success;
