@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2007-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2007-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -24,15 +24,29 @@
 extern "C" {
 #endif
 
+#include <cutter/cut-config.h>
+
 #include <stdarg.h>
 #include <stdlib.h>
 #include <setjmp.h>
-#ifdef HAVE_INTTYPES_H
+#ifdef CUT_HAVE_INTTYPES_H
 #  include <inttypes.h>
-#elif defined(HAVE_STDINT_H)
+#elif defined(CUT_HAVE_STDINT_H)
 #  include <stdint.h>
 #endif
 
+#ifndef CUT_DISABLE_SOCKET_SUPPORT
+#  ifdef CUT_HAVE_WINSOCK2_H
+#    include <winsock2.h>
+#    include <ws2tcpip.h>
+#  else
+#    include <sys/socket.h>
+#    ifdef CUT_HAVE_SYS_UN_H
+#      include <sys/un.h>
+#    endif
+#  endif
+#endif
+
 #include <cutter/cut-types.h>
 #include <cutter/cut-macros.h>
 #include <cutter/cut-multi-process.h>
@@ -88,15 +102,16 @@ cut_boolean
 void  cut_test_context_register_result      (CutTestContext *context,
                                              CutTestResultStatus status,
                                              const char *system_message);
-void         cut_test_context_set_have_user_message_jump
-                                            (CutTestContext *context,
-                                             cut_boolean     have);
-cut_boolean  cut_test_context_get_have_user_message_jump
+void         cut_test_context_start_user_message_jump
+                                            (CutTestContext *context);
+void         cut_test_context_finish_user_message_jump
+                                            (CutTestContext *context);
+cut_boolean  cut_test_context_in_user_message_jump
                                             (CutTestContext *context);
 void         cut_test_context_set_jump      (CutTestContext *context,
                                              jmp_buf        *buffer);
 jmp_buf     *cut_test_context_get_jump      (CutTestContext *context);
-void         cut_test_context_long_jump     (CutTestContext *context) CUT_GNUC_NORETURN;
+void         cut_test_context_long_jump     (CutTestContext *context);
 
 const void *cut_test_context_take           (CutTestContext *context,
                                              void           *object,
@@ -134,15 +149,24 @@ cut_boolean  cut_utils_equal_double         (double double1,
 cut_boolean  cut_utils_equal_string_array   (char **strings1,
                                              char **strings2);
 char        *cut_utils_inspect_string_array (char **strings);
-const char  *cut_utils_inspect_string       (const char *string);
+char        *cut_utils_inspect_string       (const char *string);
+
+#ifndef CUT_DISABLE_SOCKET_SUPPORT
+cut_boolean  cut_utils_equal_sockaddr       (const struct sockaddr *address1,
+                                             const struct sockaddr *address2);
+char        *cut_utils_inspect_sockaddr     (const struct sockaddr *address);
+#endif
 
 cut_boolean  cut_utils_path_exist           (const char *path);
 char        *cut_utils_build_path           (const char *path,
                                              ...) CUT_GNUC_NULL_TERMINATED;
 char        *cut_utils_build_path_va_list   (const char *path,
                                              va_list args);
+char        *cut_utils_build_path_array     (const char **paths);
 void         cut_utils_remove_path_recursive_force
                                             (const char *path);
+void         cut_utils_make_directory_recursive_force
+                                            (const char *path);
 cut_boolean  cut_utils_regex_match          (const char *pattern,
                                              const char *string);
 char        *cut_utils_fold                 (const char *string);
@@ -150,19 +174,20 @@ char        *cut_utils_append_diff          (const char *message,
                                              const char *from,
                                              const char *to);
 
-char       *cut_test_context_build_fixture_data_path
+const char *cut_test_context_build_fixture_path
                                             (CutTestContext *context,
                                              const char     *path,
                                              ...) CUT_GNUC_NULL_TERMINATED;
 
-const char *cut_utils_get_fixture_data_string
-                                            (CutTestContext *context,
-                                             char **fixture_data_path,
+const char *cut_utils_get_fixture_data      (CutTestContext *context,
+                                             const char **fixture_path,
+                                             size_t *size,
                                              const char *path,
                                              ...) CUT_GNUC_NULL_TERMINATED;
-const char *cut_utils_get_fixture_data_string_va_list
+const char *cut_utils_get_fixture_data_va_list
                                             (CutTestContext *context,
-                                             char **fixture_data_path,
+                                             const char **fixture_path,
+                                             size_t *size,
                                              const char *path,
                                              va_list args);
 
