@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2007-2009  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2007-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -30,9 +30,9 @@
 #include "cut-module-factory.h"
 #include "cut-module-factory-utils.h"
 
-static const gchar *stream_name = NULL;
+static gchar *stream_name = NULL;
 static int fd = -1;
-static const gchar *directory = NULL;
+static gchar *directory = NULL;
 static CutStreamFactoryBuilder *the_builder = NULL;
 #ifdef G_OS_WIN32
 static gchar *win32_stream_factory_module_dir = NULL;
@@ -212,13 +212,11 @@ static GList *
 build (CutFactoryBuilder *builder)
 {
     GList *factories = NULL;
-    CutStreamFactoryBuilderPrivate *priv;
     CutModuleFactory *factory;
 
     if (!stream_name)
         return NULL;
 
-    priv = CUT_STREAM_FACTORY_BUILDER_GET_PRIVATE(builder);
     factory = build_factory(builder, stream_name,
                             "fd", fd,
                             "directory", directory,
