@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -90,6 +90,10 @@ typedef enum {
     IN_RESULT_BACKTRACE_ENTRY_INFO,
     IN_RESULT_START_TIME,
     IN_RESULT_ELAPSED,
+    IN_RESULT_EXPECTED,
+    IN_RESULT_ACTUAL,
+    IN_RESULT_DIFF,
+    IN_RESULT_FOLDED_DIFF,
 
     IN_COMPLETE_ITERATED_TEST,
 
@@ -225,6 +229,8 @@ struct _CutStreamParserPrivate
     gchar *option_value;
     gboolean complete_success;
     gboolean stream_success;
+
+    GQueue *element_stack;
 };
 
 #define PUSH_STATE(priv, state)                                 \
@@ -415,6 +421,8 @@ cut_stream_parser_init (CutStreamParser *stream_parser)
     priv->option_value = NULL;
     priv->complete_success = TRUE;
     priv->stream_success = TRUE;
+
+    priv->element_stack = g_queue_new();
 }
 
 static ReadyTestSuite *
@@ -727,6 +735,12 @@ dispose (GObject *object)
         priv->option_value = NULL;
     }
 
+    if (priv->element_stack) {
+        g_queue_foreach(priv->element_stack, (GFunc)g_free, NULL);
+        g_queue_free(priv->element_stack);
+        priv->element_stack = NULL;
+    }
+
     G_OBJECT_CLASS(cut_stream_parser_parent_class)->dispose(object);
 }
 
@@ -812,26 +826,25 @@ cut_stream_parser_end_parse (CutStreamParser *stream_parser, GError **error)
 }
 
 static gchar *
-element_path (GMarkupParseContext *context)
+element_path (const GList *elements)
 {
     GString *string;
-    const GSList *node;
+    const GList *node;
 
     string = g_string_new(NULL);
-    for (node = g_markup_parse_context_get_element_stack(context);
-         node;
-         node = g_slist_next(node)) {
-        g_string_prepend(string, node->data);
-        g_string_prepend(string, "/");
+    for (node = elements; node; node = g_list_next(node)) {
+        g_string_append(string, "/");
+        g_string_append(string, node->data);
     }
 
     return g_string_free(string, FALSE);
 }
 
 static void
-set_parse_error (GMarkupParseContext *context,
-                 GError             **error,
-                 const gchar         *format, ...)
+set_parse_error (CutStreamParserPrivate *priv,
+                 GMarkupParseContext *context,
+                 GError **error,
+                 const gchar *format, ...)
 {
     gint line = 0, chr = 0;
     gchar *message, *path;
@@ -842,7 +855,7 @@ set_parse_error (GMarkupParseContext *context,
     va_end(var_args);
 
     g_markup_parse_context_get_position(context, &line, &chr);
-    path = element_path(context);
+    path = element_path(priv->element_stack->head);
     g_set_error(error, G_MARKUP_ERROR, G_MARKUP_ERROR_PARSE,
                 "Error on line %d char %d: %s: %s",
                 line, chr, path, message);
@@ -851,21 +864,10 @@ set_parse_error (GMarkupParseContext *context,
 }
 
 static void
-invalid_element (GMarkupParseContext *context, GError **error)
+invalid_element (CutStreamParserPrivate *priv,
+                 GMarkupParseContext *context, GError **error)
 {
-    set_parse_error(context, error, "invalid element");
-}
-
-static const gchar *
-get_parent_element (GMarkupParseContext *context)
-{
-    const GSList *elements, *node;
-
-    elements = g_markup_parse_context_get_element_stack(context);
-
-    node = g_slist_next(elements);
-
-    return node ? node->data : NULL;
+    set_parse_error(priv, context, error, "invalid element");
 }
 
 static void
@@ -877,7 +879,7 @@ start_top_level (CutStreamParserPrivate *priv, GMarkupParseContext *context,
         if (priv->run_context)
             g_signal_emit_by_name(priv->run_context, "start-run");
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -937,7 +939,7 @@ start_stream (CutStreamParserPrivate *priv,
     } else if (g_str_equal("success", element_name)) {
         PUSH_STATE(priv, IN_STREAM_SUCCESS);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -955,7 +957,7 @@ start_ready_test_suite (CutStreamParserPrivate *priv,
     } else if (g_str_equal("n-tests", element_name)) {
         PUSH_STATE(priv, IN_READY_TEST_SUITE_N_TESTS);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -968,7 +970,7 @@ start_start_test_suite (CutStreamParserPrivate *priv,
         PUSH_STATE(priv, IN_TEST_SUITE);
         PUSH_TEST_SUITE(priv, cut_test_suite_new_empty());
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -984,7 +986,7 @@ start_ready_test_case (CutStreamParserPrivate *priv,
     } else if (g_str_equal("n-tests", element_name)) {
         PUSH_STATE(priv, IN_READY_TEST_CASE_N_TESTS);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -997,7 +999,7 @@ start_start_test_case (CutStreamParserPrivate *priv,
         PUSH_STATE(priv, IN_TEST_CASE);
         PUSH_TEST_CASE(priv, cut_test_case_new_empty());
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1013,7 +1015,7 @@ start_ready_test_iterator (CutStreamParserPrivate *priv,
     } else if (g_str_equal("n-tests", element_name)) {
         PUSH_STATE(priv, IN_READY_TEST_ITERATOR_N_TESTS);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1026,7 +1028,7 @@ start_start_test_iterator (CutStreamParserPrivate *priv,
         PUSH_STATE(priv, IN_TEST_ITERATOR);
         PUSH_TEST_ITERATOR(priv, cut_test_iterator_new_empty());
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1044,7 +1046,7 @@ start_start_test (CutStreamParserPrivate *priv,
         priv->start_test->test_context = cut_test_context_new_empty();
         PUSH_TEST_CONTEXT(priv,  priv->start_test->test_context);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1062,7 +1064,7 @@ start_start_iterated_test (CutStreamParserPrivate *priv,
         priv->start_iterated_test->test_context = cut_test_context_new_empty();
         PUSH_TEST_CONTEXT(priv,  priv->start_iterated_test->test_context);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1084,7 +1086,7 @@ start_pass_assertion (CutStreamParserPrivate *priv,
         priv->pass_assertion->test_context = cut_test_context_new_empty();
         PUSH_TEST_CONTEXT(priv,  priv->pass_assertion->test_context);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1110,7 +1112,7 @@ start_test_result (CutStreamParserPrivate *priv,
         priv->test_result->result = cut_test_result_new_empty();
         priv->result = g_object_ref(priv->test_result->result);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1123,7 +1125,7 @@ start_top_level_result (CutStreamParserPrivate *priv,
         PUSH_STATE(priv, IN_RESULT);
         priv->result = cut_test_result_new_empty();
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1177,8 +1179,16 @@ start_result (CutStreamParserPrivate *priv, GMarkupParseContext *context,
         PUSH_STATE(priv, IN_RESULT_START_TIME);
     } else if (g_str_equal("elapsed", element_name)) {
         PUSH_STATE(priv, IN_RESULT_ELAPSED);
+    } else if (g_str_equal("expected", element_name)) {
+        PUSH_STATE(priv, IN_RESULT_EXPECTED);
+    } else if (g_str_equal("actual", element_name)) {
+        PUSH_STATE(priv, IN_RESULT_ACTUAL);
+    } else if (g_str_equal("diff", element_name)) {
+        PUSH_STATE(priv, IN_RESULT_DIFF);
+    } else if (g_str_equal("folded-diff", element_name)) {
+        PUSH_STATE(priv, IN_RESULT_FOLDED_DIFF);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1200,7 +1210,7 @@ start_complete_iterated_test (CutStreamParserPrivate *priv,
     } else if (g_str_equal("success", element_name)) {
         PUSH_STATE(priv, IN_COMPLETE_SUCCESS);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1219,7 +1229,7 @@ start_complete_test (CutStreamParserPrivate *priv, GMarkupParseContext *context,
     } else if (g_str_equal("success", element_name)) {
         PUSH_STATE(priv, IN_COMPLETE_SUCCESS);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1238,7 +1248,7 @@ start_test_iterator_result (CutStreamParserPrivate *priv,
         priv->test_iterator_result->result = cut_test_result_new_empty();
         priv->result = g_object_ref(priv->test_iterator_result->result);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1253,7 +1263,7 @@ start_complete_test_iterator (CutStreamParserPrivate *priv,
     } else if (g_str_equal("success", element_name)) {
         PUSH_STATE(priv, IN_COMPLETE_SUCCESS);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1271,7 +1281,7 @@ start_test_case_result (CutStreamParserPrivate *priv,
         priv->test_case_result->result = cut_test_result_new_empty();
         priv->result = g_object_ref(priv->test_case_result->result);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1286,7 +1296,7 @@ start_complete_test_case (CutStreamParserPrivate *priv,
     } else if (g_str_equal("success", element_name)) {
         PUSH_STATE(priv, IN_COMPLETE_SUCCESS);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1301,7 +1311,7 @@ start_complete_test_suite (CutStreamParserPrivate *priv,
     } else if (g_str_equal("success", element_name)) {
         PUSH_STATE(priv, IN_COMPLETE_SUCCESS);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1320,7 +1330,7 @@ start_test_object (CutStreamParserPrivate *priv, GMarkupParseContext *context,
     } else if (g_str_equal("elapsed", element_name)) {
         PUSH_STATE(priv, IN_TEST_ELAPSED);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1333,7 +1343,7 @@ start_test_option (CutStreamParserPrivate *priv, GMarkupParseContext *context,
     } else if (g_str_equal("value", element_name)) {
         PUSH_STATE(priv, IN_TEST_OPTION_VALUE);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1346,7 +1356,7 @@ start_result_backtrace (CutStreamParserPrivate *priv,
         PUSH_STATE(priv, IN_RESULT_BACKTRACE_ENTRY);
         priv->backtrace_entry = cut_backtrace_entry_new_empty();
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1362,7 +1372,7 @@ start_result_backtrace_entry (CutStreamParserPrivate *priv,
     } else if (g_str_equal("info", element_name)) {
         PUSH_STATE(priv, IN_RESULT_BACKTRACE_ENTRY_INFO);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1423,7 +1433,7 @@ start_test_context (CutStreamParserPrivate *priv, GMarkupParseContext *context,
     } else if (g_str_equal("failed", element_name)) {
         PUSH_STATE(priv, IN_TEST_CONTEXT_FAILED);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1434,7 +1444,7 @@ start_test_data (CutStreamParserPrivate *priv, GMarkupParseContext *context,
     if (g_str_equal("name", element_name)) {
         PUSH_STATE(priv, IN_TEST_DATA_NAME);
     } else {
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
     }
 }
 
@@ -1451,6 +1461,7 @@ start_element_handler (GMarkupParseContext *context,
     ParseState state;
 
     priv = CUT_STREAM_PARSER_GET_PRIVATE(parser);
+    g_queue_push_tail(priv->element_stack, g_strdup(element_name));
 
     state = PEEK_STATE(priv);
     switch (state) {
@@ -1540,7 +1551,7 @@ start_element_handler (GMarkupParseContext *context,
         start_test_data(priv, context, element_name, error);
         break;
       default:
-        invalid_element(context, error);
+        invalid_element(priv, context, error);
         break;
     }
 }
@@ -1602,9 +1613,9 @@ end_test_option (CutStreamParser *parser, CutStreamParserPrivate *priv,
                  const gchar *element_name, GError **error)
 {
     if (!priv->option_name) {
-        set_parse_error(context, error, "option name is not set");
+        set_parse_error(priv, context, error, "option name is not set");
     } else if (!priv->option_value) {
-        set_parse_error(context, error, "option value is not set");
+        set_parse_error(priv, context, error, "option value is not set");
     } else {
         cut_test_set_attribute(PEEK_TEST(priv),
                                priv->option_name,
@@ -2040,89 +2051,89 @@ end_element_handler (GMarkupParseContext *context,
 {
     CutStreamParser *parser = user_data;
     CutStreamParserPrivate *priv;
-    const gchar *parent_name;
     ParseState state;
 
     priv = CUT_STREAM_PARSER_GET_PRIVATE(parser);
 
-    parent_name = get_parent_element(context);
     state = POP_STATE(priv);
     switch (state) {
-      case IN_TOP_LEVEL_RESULT:
+    case IN_TOP_LEVEL_RESULT:
         end_top_level_result(parser, priv, context, element_name, error);
         break;
-      case IN_STREAM:
+    case IN_STREAM:
         end_stream(parser, priv, context, element_name, error);
         break;
-      case IN_RESULT:
+    case IN_RESULT:
         end_result(parser, priv, context, element_name, error);
         break;
-      case IN_TEST_CONTEXT:
+    case IN_TEST_CONTEXT:
         end_test_context(parser, priv, context, element_name, error);
         break;
-      case IN_TEST_OPTION:
+    case IN_TEST_OPTION:
         end_test_option(parser, priv, context, element_name, error);
         break;
-      case IN_RESULT_BACKTRACE:
+    case IN_RESULT_BACKTRACE:
         end_result_backtrace(parser, priv, context, element_name, error);
         break;
-      case IN_RESULT_BACKTRACE_ENTRY:
+    case IN_RESULT_BACKTRACE_ENTRY:
         end_result_backtrace_entry(parser, priv, context, element_name, error);
         break;
-      case IN_READY_TEST_SUITE:
+    case IN_READY_TEST_SUITE:
         end_ready_test_suite(parser, priv, context, element_name, error);
         break;
-      case IN_START_TEST_SUITE:
+    case IN_START_TEST_SUITE:
         end_start_test_suite(parser, priv, context, element_name, error);
         break;
-      case IN_READY_TEST_CASE:
+    case IN_READY_TEST_CASE:
         end_ready_test_case(parser, priv, context, element_name, error);
         break;
-      case IN_START_TEST_CASE:
+    case IN_START_TEST_CASE:
         end_start_test_case(parser, priv, context, element_name, error);
         break;
-      case IN_READY_TEST_ITERATOR:
+    case IN_READY_TEST_ITERATOR:
         end_ready_test_iterator(parser, priv, context, element_name, error);
         break;
-      case IN_START_TEST_ITERATOR:
+    case IN_START_TEST_ITERATOR:
         end_start_test_iterator(parser, priv, context, element_name, error);
         break;
-      case IN_START_TEST:
+    case IN_START_TEST:
         end_start_test(parser, priv, context, element_name, error);
         break;
-      case IN_START_ITERATED_TEST:
+    case IN_START_ITERATED_TEST:
         end_start_iterated_test(parser, priv, context, element_name, error);
         break;
-      case IN_PASS_ASSERTION:
+    case IN_PASS_ASSERTION:
         end_pass_assertion(parser, priv, context, element_name, error);
         break;
-      case IN_TEST_RESULT:
+    case IN_TEST_RESULT:
         end_test_result(parser, priv, context, element_name, error);
         break;
-      case IN_COMPLETE_ITERATED_TEST:
+    case IN_COMPLETE_ITERATED_TEST:
         end_complete_iterated_test(parser, priv, context, element_name, error);
         break;
-      case IN_COMPLETE_TEST:
+    case IN_COMPLETE_TEST:
         end_complete_test(parser, priv, context, element_name, error);
         break;
-      case IN_TEST_ITERATOR_RESULT:
+    case IN_TEST_ITERATOR_RESULT:
         end_test_iterator_result(parser, priv, context, element_name, error);
         break;
-      case IN_COMPLETE_TEST_ITERATOR:
+    case IN_COMPLETE_TEST_ITERATOR:
         end_complete_test_iterator(parser, priv, context, element_name, error);
         break;
-      case IN_TEST_CASE_RESULT:
+    case IN_TEST_CASE_RESULT:
         end_test_case_result(parser, priv, context, element_name, error);
         break;
-      case IN_COMPLETE_TEST_CASE:
+    case IN_COMPLETE_TEST_CASE:
         end_complete_test_case(parser, priv, context, element_name, error);
         break;
-      case IN_COMPLETE_TEST_SUITE:
+    case IN_COMPLETE_TEST_SUITE:
         end_complete_test_suite(parser, priv, context, element_name, error);
         break;
-      default:
+    default:
         break;
     }
+
+    g_free(g_queue_pop_tail(priv->element_stack));
 }
 
 static CutTestResultStatus
@@ -2214,7 +2225,7 @@ text_test_name (CutStreamParserPrivate *priv, GMarkupParseContext *context,
     if (target) {
         cut_test_set_name(target, text);
     } else {
-        set_parse_error(context, error, "can't find test name target");
+        set_parse_error(priv, context, error, "can't find test name target");
     }
 }
 
@@ -2229,7 +2240,8 @@ text_test_description (CutStreamParserPrivate *priv,
     if (target) {
         cut_test_set_attribute(target, "description", text);
     } else {
-        set_parse_error(context, error, "can't find test description target");
+        set_parse_error(priv, context, error,
+                        "can't find test description target");
     }
 }
 
@@ -2239,7 +2251,7 @@ text_test_option_name (CutStreamParserPrivate *priv,
                        const gchar *text, gsize text_len, GError **error)
 {
     if (priv->option_name) {
-        set_parse_error(context, error, "multiple option name: %s", text);
+        set_parse_error(priv, context, error, "multiple option name: %s", text);
     } else {
         priv->option_name = g_strdup(text);
     }
@@ -2251,7 +2263,7 @@ text_test_option_value (CutStreamParserPrivate *priv,
                         const gchar *text, gsize text_len, GError **error)
 {
     if (priv->option_value) {
-        set_parse_error(context, error, "multiple option value: %s", text);
+        set_parse_error(priv, context, error, "multiple option value: %s", text);
     } else {
         priv->option_value = g_strdup(text);
     }
@@ -2271,12 +2283,13 @@ text_test_start_time (CutStreamParserPrivate *priv,
         if (g_time_val_from_iso8601(text, &start_time)) {
             cut_test_set_start_time(target, &start_time);
         } else {
-            set_parse_error(context, error,
+            set_parse_error(priv, context, error,
                             "invalid start-time value (not ISO 8601 format): %s",
                             text);
         }
     } else {
-        set_parse_error(context, error, "can't find test start time target");
+        set_parse_error(priv, context, error,
+                        "can't find test start time target");
     }
 }
 
@@ -2296,10 +2309,11 @@ text_test_elapsed (CutStreamParserPrivate *priv,
         if (text != end_position && end_position[0] == '\0') {
             cut_test_set_elapsed(target, elapsed);
         } else {
-            set_parse_error(context, error, "invalid elapsed value: %s", text);
+            set_parse_error(priv, context, error,
+                            "invalid elapsed value: %s", text);
         }
     } else {
-        set_parse_error(context, error, "can't find test elapsed target");
+        set_parse_error(priv, context, error, "can't find test elapsed target");
     }
 }
 
@@ -2311,7 +2325,7 @@ text_result_status (CutStreamParserPrivate *priv, GMarkupParseContext *context,
 
     status = result_name_to_status(text);
     if (status == CUT_TEST_RESULT_INVALID) {
-        set_parse_error(context, error, "invalid status: %s", text);
+        set_parse_error(priv, context, error, "invalid status: %s", text);
     } else {
         cut_test_result_set_status(priv->result, status);
     }
@@ -2342,7 +2356,7 @@ text_result_backtrace_entry_line (CutStreamParserPrivate *priv,
     if (is_integer(text)) {
         cut_backtrace_entry_set_line(priv->backtrace_entry, atoi(text));
     } else {
-        set_parse_error(context, error, "invalid line number: %s", text);
+        set_parse_error(priv, context, error, "invalid line number: %s", text);
     }
 }
 
@@ -2387,7 +2401,7 @@ text_result_start_time (CutStreamParserPrivate *priv,
     if (g_time_val_from_iso8601(text, &start_time)) {
         cut_test_result_set_start_time(priv->result, &start_time);
     } else {
-        set_parse_error(context, error,
+        set_parse_error(priv, context, error,
                         "invalid start-time value (not ISO 8601 format): %s",
                         text);
     }
@@ -2404,10 +2418,39 @@ text_result_elapsed (CutStreamParserPrivate *priv, GMarkupParseContext *context,
     if (text != end_position && end_position[0] == '\0') {
         cut_test_result_set_elapsed(priv->result, elapsed);
     } else {
-        set_parse_error(context, error, "invalid elapsed value: %s", text);
+        set_parse_error(priv, context, error, "invalid elapsed value: %s", text);
     }
 }
 
+static void
+text_result_expected (CutStreamParserPrivate *priv, GMarkupParseContext *context,
+                      const gchar *text, gsize text_len, GError **error)
+{
+    cut_test_result_set_expected(priv->result, text);
+}
+
+static void
+text_result_actual (CutStreamParserPrivate *priv, GMarkupParseContext *context,
+                    const gchar *text, gsize text_len, GError **error)
+{
+    cut_test_result_set_actual(priv->result, text);
+}
+
+static void
+text_result_diff (CutStreamParserPrivate *priv, GMarkupParseContext *context,
+                  const gchar *text, gsize text_len, GError **error)
+{
+    cut_test_result_set_diff(priv->result, text);
+}
+
+static void
+text_result_folded_diff (CutStreamParserPrivate *priv,
+                         GMarkupParseContext *context,
+                         const gchar *text, gsize text_len, GError **error)
+{
+    cut_test_result_set_folded_diff(priv->result, text);
+}
+
 static void
 text_ready_test_suite_n_test_cases (CutStreamParserPrivate *priv,
                                     GMarkupParseContext *context,
@@ -2417,7 +2460,8 @@ text_ready_test_suite_n_test_cases (CutStreamParserPrivate *priv,
     if (is_integer(text)) {
         priv->ready_test_suite->n_test_cases = atoi(text);
     } else {
-        set_parse_error(context, error, "invalid # of test cases: %s", text);
+        set_parse_error(priv, context, error,
+                        "invalid # of test cases: %s", text);
     }
 }
 
@@ -2430,7 +2474,7 @@ text_ready_test_suite_n_tests (CutStreamParserPrivate *priv,
     if (is_integer(text)) {
         priv->ready_test_suite->n_tests = atoi(text);
     } else {
-        set_parse_error(context, error, "invalid # of tests: %s", text);
+        set_parse_error(priv, context, error, "invalid # of tests: %s", text);
     }
 }
 
@@ -2442,7 +2486,7 @@ text_ready_test_case_n_tests (CutStreamParserPrivate *priv,
     if (is_integer(text)) {
         priv->ready_test_case->n_tests = atoi(text);
     } else {
-        set_parse_error(context, error, "invalid # of tests: %s", text);
+        set_parse_error(priv, context, error, "invalid # of tests: %s", text);
     }
 }
 
@@ -2455,7 +2499,7 @@ text_ready_test_iterator_n_tests (CutStreamParserPrivate *priv,
     if (is_integer(text)) {
         priv->ready_test_iterator->n_tests = atoi(text);
     } else {
-        set_parse_error(context, error, "invalid # of tests: %s", text);
+        set_parse_error(priv, context, error, "invalid # of tests: %s", text);
     }
 }
 
@@ -2479,7 +2523,7 @@ text_test_context_failed (CutStreamParserPrivate *priv,
         cut_test_context_set_failed(PEEK_TEST_CONTEXT(priv),
                                     string_to_boolean(text));
     } else {
-        set_parse_error(context, error, "invalid boolean value: %s", text);
+        set_parse_error(priv, context, error, "invalid boolean value: %s", text);
     }
 }
 
@@ -2491,7 +2535,7 @@ text_complete_success (CutStreamParserPrivate *priv,
     if (is_boolean(text)) {
         priv->complete_success = string_to_boolean(text);
     } else {
-        set_parse_error(context, error, "invalid boolean value: %s", text);
+        set_parse_error(priv, context, error, "invalid boolean value: %s", text);
     }
 }
 
@@ -2502,7 +2546,7 @@ text_stream_success (CutStreamParserPrivate *priv, GMarkupParseContext *context,
     if (is_boolean(text)) {
         priv->stream_success = string_to_boolean(text);
     } else {
-        set_parse_error(context, error, "invalid boolean value: %s", text);
+        set_parse_error(priv, context, error, "invalid boolean value: %s", text);
     }
 }
 
@@ -2515,77 +2559,87 @@ text_handler (GMarkupParseContext *context,
 {
     CutStreamParser *parser = user_data;
     CutStreamParserPrivate *priv;
-    const gchar *element;
     ParseState state;
 
     priv = CUT_STREAM_PARSER_GET_PRIVATE(parser);
-    element = g_markup_parse_context_get_element(context);
 
     state = PEEK_STATE(priv);
     switch (state) {
-      case IN_TEST_NAME:
+    case IN_TEST_NAME:
         text_test_name(priv, context, text, text_len, error);
         break;
-      case IN_TEST_DESCRIPTION:
+    case IN_TEST_DESCRIPTION:
         text_test_description(priv, context, text, text_len, error);
         break;
-      case IN_TEST_OPTION_NAME:
+    case IN_TEST_OPTION_NAME:
         text_test_option_name(priv, context, text, text_len, error);
         break;
-      case IN_TEST_OPTION_VALUE:
+    case IN_TEST_OPTION_VALUE:
         text_test_option_value(priv, context, text, text_len, error);
         break;
-      case IN_TEST_START_TIME:
+    case IN_TEST_START_TIME:
         text_test_start_time(priv, context, text, text_len, error);
         break;
-      case IN_TEST_ELAPSED:
+    case IN_TEST_ELAPSED:
         text_test_elapsed(priv, context, text, text_len, error);
         break;
-      case IN_RESULT_STATUS:
+    case IN_RESULT_STATUS:
         text_result_status(priv, context, text, text_len, error);
         break;
-      case IN_RESULT_DETAIL:
+    case IN_RESULT_DETAIL:
         text_result_detail(priv, context, text, text_len, error);
         break;
-      case IN_RESULT_BACKTRACE_ENTRY_FILE:
+    case IN_RESULT_BACKTRACE_ENTRY_FILE:
         text_result_backtrace_entry_file(priv, context, text, text_len, error);
         break;
-      case IN_RESULT_BACKTRACE_ENTRY_LINE:
+    case IN_RESULT_BACKTRACE_ENTRY_LINE:
         text_result_backtrace_entry_line(priv, context, text, text_len, error);
         break;
-      case IN_RESULT_BACKTRACE_ENTRY_INFO:
+    case IN_RESULT_BACKTRACE_ENTRY_INFO:
         text_result_backtrace_entry_info(priv, context, text, text_len, error);
         break;
-      case IN_RESULT_START_TIME:
+    case IN_RESULT_START_TIME:
         text_result_start_time(priv, context, text, text_len, error);
         break;
-      case IN_RESULT_ELAPSED:
+    case IN_RESULT_ELAPSED:
         text_result_elapsed(priv, context, text, text_len, error);
         break;
-      case IN_READY_TEST_SUITE_N_TEST_CASES:
+    case IN_RESULT_EXPECTED:
+        text_result_expected(priv, context, text, text_len, error);
+        break;
+    case IN_RESULT_ACTUAL:
+        text_result_actual(priv, context, text, text_len, error);
+        break;
+    case IN_RESULT_DIFF:
+        text_result_diff(priv, context, text, text_len, error);
+        break;
+    case IN_RESULT_FOLDED_DIFF:
+        text_result_folded_diff(priv, context, text, text_len, error);
+        break;
+    case IN_READY_TEST_SUITE_N_TEST_CASES:
         text_ready_test_suite_n_test_cases(priv, context, text, text_len, error);
         break;
-      case IN_READY_TEST_SUITE_N_TESTS:
+    case IN_READY_TEST_SUITE_N_TESTS:
         text_ready_test_suite_n_tests(priv, context, text, text_len, error);
         break;
-      case IN_READY_TEST_CASE_N_TESTS:
+    case IN_READY_TEST_CASE_N_TESTS:
         text_ready_test_case_n_tests(priv, context, text, text_len, error);
         break;
-      case IN_READY_TEST_ITERATOR_N_TESTS:
+    case IN_READY_TEST_ITERATOR_N_TESTS:
         text_ready_test_iterator_n_tests(priv, context, text, text_len, error);
         break;
-      case IN_TEST_DATA_NAME:
+    case IN_TEST_DATA_NAME:
         text_test_data_name(priv, context, text, text_len, error);
         break;
-      case IN_TEST_CONTEXT_FAILED:
+    case IN_TEST_CONTEXT_FAILED:
         text_test_context_failed(priv, context, text, text_len, error);
         break;
-      case IN_COMPLETE_SUCCESS:
+    case IN_COMPLETE_SUCCESS:
         text_complete_success(priv, context, text, text_len, error);
-      case IN_STREAM_SUCCESS:
+    case IN_STREAM_SUCCESS:
         text_stream_success(priv, context, text, text_len, error);
         break;
-      default:
+    default:
         break;
     }
 }
