@@ -28,9 +28,7 @@
 #ifdef HAVE_SYS_WAIT_H
 #  include <sys/wait.h>
 #endif
-#ifdef HAVE_UNISTD_H
-#  include <unistd.h>
-#endif
+#include <signal.h>
 #include <glib.h>
 #include <glib/gi18n-lib.h>
 
@@ -41,6 +39,10 @@
 #  include <winsock2.h>
 #  include <process.h>		/* For getpid() */
 #  include <io.h>
+#else
+#  ifdef HAVE_UNISTD_H
+#    include <unistd.h>
+#  endif
 #endif
 
 #include "cut-run-context.h"
@@ -854,8 +856,11 @@ cut_test_context_long_jump (CutTestContext *context)
     }
 
     if (cut_test_context_in_user_message_jump(context) || same_thread_p) {
-        longjmp(*(priv->jump_buffer), 1);
+        longjmp(*(priv->jump_buffer), SIGCHLD);
     } else {
+        if (priv->parent && cut_test_context_is_failed(context)) {
+            cut_test_context_set_failed(priv->parent, TRUE);
+        }
         cut_test_context_current_pop();
         g_object_unref(context);
         g_thread_exit(NULL);
