@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2007-2009  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2007-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -34,6 +34,9 @@
 #include "cut-stream-parser.h"
 #include "cut-backtrace-entry.h"
 #include "cut-utils.h"
+#include "cut-readable-differ.h"
+
+#define MAX_DIFF_TARGET_SIZE 8092
 
 #define CUT_TEST_RESULT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CUT_TYPE_TEST_RESULT, CutTestResultPrivate))
 
@@ -47,11 +50,18 @@ struct _CutTestResultPrivate
     CutTestSuite *test_suite;
     CutTestData *test_data;
     gchar *message;
+    gboolean user_set_message;
     gchar *user_message;
     gchar *system_message;
     GList *backtrace;
     GTimeVal start_time;
     gdouble elapsed;
+    gchar *expected;
+    gchar *actual;
+    gchar *diff;
+    gchar *folded_diff;
+    gboolean user_set_diff;
+    gboolean user_set_folded_diff;
 };
 
 enum
@@ -67,6 +77,10 @@ enum
     PROP_SYSTEM_MESSAGE,
     PROP_BACKTRACE,
     PROP_ELAPSED,
+    PROP_EXPECTED,
+    PROP_ACTUAL,
+    PROP_DIFF,
+    PROP_FOLDED_DIFF,
 };
 
 
@@ -164,6 +178,36 @@ cut_test_result_class_init (CutTestResultClass *klass)
                                G_PARAM_READWRITE);
     g_object_class_install_property(gobject_class, PROP_ELAPSED, spec);
 
+    spec = g_param_spec_string("expected",
+                               "Expected object",
+                               "The inspected string of expected object",
+                               NULL,
+                               G_PARAM_READWRITE);
+    g_object_class_install_property(gobject_class, PROP_EXPECTED, spec);
+
+    spec = g_param_spec_string("actual",
+                               "Actual object",
+                               "The inspected string of actual object",
+                               NULL,
+                               G_PARAM_READWRITE);
+    g_object_class_install_property(gobject_class, PROP_ACTUAL, spec);
+
+    spec = g_param_spec_string("diff",
+                               "Difference",
+                               "The difference between expected object "
+                               "and actual object",
+                               NULL,
+                               G_PARAM_READWRITE);
+    g_object_class_install_property(gobject_class, PROP_DIFF, spec);
+
+    spec = g_param_spec_string("folded-diff",
+                               "Folded difference",
+                               "The difference between folded expected object "
+                               "and folded actual object",
+                               NULL,
+                               G_PARAM_READWRITE);
+    g_object_class_install_property(gobject_class, PROP_FOLDED_DIFF, spec);
+
     g_type_class_add_private(gobject_class, sizeof(CutTestResultPrivate));
 }
 
@@ -179,12 +223,19 @@ cut_test_result_init (CutTestResult *result)
     priv->test_suite = NULL;
     priv->test_data = NULL;
     priv->message = NULL;
+    priv->user_set_message = FALSE;
     priv->user_message = NULL;
     priv->system_message = NULL;
     priv->backtrace = NULL;
     priv->start_time.tv_sec = 0;
     priv->start_time.tv_usec = 0;
     priv->elapsed = 0.0;
+    priv->expected = NULL;
+    priv->actual = NULL;
+    priv->diff = NULL;
+    priv->folded_diff= NULL;
+    priv->user_set_diff = FALSE;
+    priv->user_set_folded_diff = FALSE;
 }
 
 static void
@@ -238,6 +289,26 @@ dispose (GObject *object)
         priv->backtrace = NULL;
     }
 
+    if (priv->expected) {
+        g_free(priv->expected);
+        priv->expected = NULL;
+    }
+
+    if (priv->actual) {
+        g_free(priv->actual);
+        priv->actual = NULL;
+    }
+
+    if (priv->diff) {
+        g_free(priv->diff);
+        priv->diff = NULL;
+    }
+
+    if (priv->folded_diff) {
+        g_free(priv->folded_diff);
+        priv->folded_diff = NULL;
+    }
+
     G_OBJECT_CLASS(cut_test_result_parent_class)->dispose(object);
 }
 
@@ -247,48 +318,57 @@ set_property (GObject      *object,
               const GValue *value,
               GParamSpec   *pspec)
 {
-    CutTestResultPrivate *priv = CUT_TEST_RESULT_GET_PRIVATE(object);
+    CutTestResult *result;
+    CutTestResultPrivate *priv;
+
+    result = CUT_TEST_RESULT(object);
+    priv = CUT_TEST_RESULT_GET_PRIVATE(result);
 
     switch (prop_id) {
-      case PROP_STATUS:
+    case PROP_STATUS:
         priv->status = g_value_get_enum(value);
         break;
-      case PROP_TEST:
-        cut_test_result_set_test(CUT_TEST_RESULT(object),
-                                 g_value_get_object(value));
+    case PROP_TEST:
+        cut_test_result_set_test(result, g_value_get_object(value));
         break;
-      case PROP_TEST_ITERATOR:
-        cut_test_result_set_test_iterator(CUT_TEST_RESULT(object),
+    case PROP_TEST_ITERATOR:
+        cut_test_result_set_test_iterator(result,
                                           g_value_get_object(value));
         break;
-      case PROP_TEST_CASE:
-        cut_test_result_set_test_case(CUT_TEST_RESULT(object),
-                                      g_value_get_object(value));
+    case PROP_TEST_CASE:
+        cut_test_result_set_test_case(result, g_value_get_object(value));
         break;
-      case PROP_TEST_SUITE:
-        cut_test_result_set_test_suite(CUT_TEST_RESULT(object),
-                                       g_value_get_object(value));
+    case PROP_TEST_SUITE:
+        cut_test_result_set_test_suite(result, g_value_get_object(value));
         break;
-      case PROP_TEST_DATA:
-        cut_test_result_set_test_data(CUT_TEST_RESULT(object),
-                                      g_value_get_object(value));
+    case PROP_TEST_DATA:
+        cut_test_result_set_test_data(result, g_value_get_object(value));
         break;
-      case PROP_USER_MESSAGE:
-        cut_test_result_set_user_message(CUT_TEST_RESULT(object),
-                                         g_value_get_string(value));
+    case PROP_USER_MESSAGE:
+        cut_test_result_set_user_message(result, g_value_get_string(value));
         break;
-      case PROP_SYSTEM_MESSAGE:
-        cut_test_result_set_system_message(CUT_TEST_RESULT(object),
-                                           g_value_get_string(value));
+    case PROP_SYSTEM_MESSAGE:
+        cut_test_result_set_system_message(result, g_value_get_string(value));
         break;
-      case PROP_BACKTRACE:
-        cut_test_result_set_backtrace(CUT_TEST_RESULT(object),
-                                      g_value_get_pointer(value));
+    case PROP_BACKTRACE:
+        cut_test_result_set_backtrace(result, g_value_get_pointer(value));
         break;
-      case PROP_ELAPSED:
+    case PROP_ELAPSED:
         priv->elapsed = g_value_get_double(value);
         break;
-      default:
+    case PROP_EXPECTED:
+        cut_test_result_set_expected(result, g_value_get_string(value));
+        break;
+    case PROP_ACTUAL:
+        cut_test_result_set_actual(result, g_value_get_string(value));
+        break;
+    case PROP_DIFF:
+        cut_test_result_set_diff(result, g_value_get_string(value));
+        break;
+    case PROP_FOLDED_DIFF:
+        cut_test_result_set_folded_diff(result, g_value_get_string(value));
+        break;
+    default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
     }
@@ -300,40 +380,56 @@ get_property (GObject    *object,
               GValue     *value,
               GParamSpec *pspec)
 {
-    CutTestResultPrivate *priv = CUT_TEST_RESULT_GET_PRIVATE(object);
+    CutTestResult *result;
+    CutTestResultPrivate *priv;
+
+    result = CUT_TEST_RESULT(object);
+    priv = CUT_TEST_RESULT_GET_PRIVATE(result);
 
     switch (prop_id) {
-      case PROP_STATUS:
+    case PROP_STATUS:
         g_value_set_enum(value, priv->status);
         break;
-      case PROP_TEST:
+    case PROP_TEST:
         g_value_set_object(value, G_OBJECT(priv->test));
         break;
-      case PROP_TEST_ITERATOR:
+    case PROP_TEST_ITERATOR:
         g_value_set_object(value, G_OBJECT(priv->test_iterator));
         break;
-      case PROP_TEST_CASE:
+    case PROP_TEST_CASE:
         g_value_set_object(value, G_OBJECT(priv->test_case));
         break;
-      case PROP_TEST_SUITE:
+    case PROP_TEST_SUITE:
         g_value_set_object(value, G_OBJECT(priv->test_suite));
         break;
-      case PROP_TEST_DATA:
+    case PROP_TEST_DATA:
         g_value_set_object(value, G_OBJECT(priv->test_data));
         break;
-      case PROP_USER_MESSAGE:
+    case PROP_USER_MESSAGE:
         g_value_set_string(value, priv->user_message);
         break;
-      case PROP_SYSTEM_MESSAGE:
+    case PROP_SYSTEM_MESSAGE:
         g_value_set_string(value, priv->system_message);
         break;
-      case PROP_BACKTRACE:
+    case PROP_BACKTRACE:
         g_value_set_pointer(value, priv->backtrace);
         break;
-      case PROP_ELAPSED:
+    case PROP_ELAPSED:
         g_value_set_double(value, priv->elapsed);
         break;
-      default:
+    case PROP_EXPECTED:
+        g_value_set_string(value, priv->expected);
+        break;
+    case PROP_ACTUAL:
+        g_value_set_string(value, priv->actual);
+        break;
+    case PROP_DIFF:
+        g_value_set_string(value, cut_test_result_get_diff(result));
+        break;
+    case PROP_FOLDED_DIFF:
+        g_value_set_string(value, cut_test_result_get_folded_diff(result));
+        break;
+    default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
     }
@@ -410,20 +506,69 @@ cut_test_result_get_status (CutTestResult *result)
     return CUT_TEST_RESULT_GET_PRIVATE(result)->status;
 }
 
+static void
+reset_message (CutTestResultPrivate *priv)
+{
+    if (priv->user_set_message)
+        return;
+
+    if (priv->message) {
+        g_free(priv->message);
+        priv->message = NULL;
+    }
+}
+
 const gchar *
 cut_test_result_get_message (CutTestResult *result)
 {
     CutTestResultPrivate *priv = CUT_TEST_RESULT_GET_PRIVATE(result);
 
-    if (!priv->message && (priv->user_message || priv->system_message)) {
-        if (priv->user_message && priv->system_message) {
-            priv->message = g_strdup_printf("%s\n%s",
-                                            priv->user_message,
-                                            priv->system_message);
+    if (!priv->message) {
+        GString *message;
+        const gchar *diff, *folded_diff;
+
+        message = g_string_new(NULL);
+        if (priv->user_message)
+            g_string_append(message, priv->user_message);
+
+        if (priv->system_message) {
+            if (message->len > 0)
+                g_string_append(message, "\n");
+            g_string_append(message, priv->system_message);
+        }
+
+        if (priv->expected) {
+            if (message->len > 0)
+                g_string_append(message, "\n");
+            g_string_append_printf(message, "expected: <%s>", priv->expected);
+        }
+
+        if (priv->actual) {
+            if (message->len > 0)
+                g_string_append(message, "\n");
+            g_string_append_printf(message, "  actual: <%s>", priv->actual);
+        }
+
+        diff = cut_test_result_get_diff(result);
+        if (diff) {
+            if (message->len > 0)
+                g_string_append(message, "\n\n");
+            g_string_append(message, "diff:\n");
+            g_string_append(message, diff);
+        }
+
+        folded_diff = cut_test_result_get_folded_diff(result);
+        if (folded_diff) {
+            if (message->len > 0)
+                g_string_append(message, "\n\n");
+            g_string_append(message, "folded diff:\n");
+            g_string_append(message, folded_diff);
+        }
+
+        if (message->len > 0) {
+            priv->message = g_string_free(message, FALSE);
         } else {
-            priv->message = g_strdup(priv->user_message ?
-                                     priv->user_message :
-                                     priv->system_message);
+            g_string_free(message, TRUE);
         }
     }
 
@@ -534,6 +679,61 @@ cut_test_result_get_elapsed (CutTestResult *result)
     return CUT_TEST_RESULT_GET_PRIVATE(result)->elapsed;
 }
 
+const gchar *
+cut_test_result_get_expected (CutTestResult *result)
+{
+    return CUT_TEST_RESULT_GET_PRIVATE(result)->expected;
+}
+
+const gchar *
+cut_test_result_get_actual (CutTestResult *result)
+{
+    return CUT_TEST_RESULT_GET_PRIVATE(result)->actual;
+}
+
+const gchar *
+cut_test_result_get_diff (CutTestResult *result)
+{
+    CutTestResultPrivate *priv;
+
+    priv = CUT_TEST_RESULT_GET_PRIVATE(result);
+
+    if (priv->diff)
+        return priv->diff;
+
+    if (priv->expected && priv->actual &&
+        strlen(priv->expected) < MAX_DIFF_TARGET_SIZE &&
+        strlen(priv->actual) < MAX_DIFF_TARGET_SIZE) {
+        priv->diff = cut_diff_readable(priv->expected, priv->actual);
+        if (!cut_diff_readable_is_interested(priv->diff)) {
+            g_free(priv->diff);
+            priv->diff = NULL;
+        }
+    }
+
+    return priv->diff;
+}
+
+const gchar *
+cut_test_result_get_folded_diff (CutTestResult *result)
+{
+    CutTestResultPrivate *priv;
+    const gchar *diff;
+
+    priv = CUT_TEST_RESULT_GET_PRIVATE(result);
+
+    if (priv->folded_diff)
+        return priv->folded_diff;
+
+    diff = cut_test_result_get_diff(result);
+    if (cut_diff_readable_need_fold(diff)) {
+        priv->folded_diff =
+            cut_diff_readable_folded(priv->expected, priv->actual);
+    }
+
+    return priv->folded_diff;
+}
+
 gchar *
 cut_test_result_to_xml (CutTestResult *result)
 {
@@ -603,10 +803,14 @@ append_test_result_to_string (GString *string, CutTestResult *result,
     CutTestResultStatus status;
     GTimeVal start_time;
     gchar *elapsed_string, *start_time_string;
-    const gchar *message;
+    const gchar *message, *expected, *actual, *diff, *folded_diff;
 
-    message = cut_test_result_get_message(result);
     status = cut_test_result_get_status(result);
+    message = cut_test_result_get_message(result);
+    expected = cut_test_result_get_expected(result);
+    actual = cut_test_result_get_actual(result);
+    diff = cut_test_result_get_diff(result);
+    folded_diff = cut_test_result_get_folded_diff(result);
 
     cut_utils_append_xml_element_with_value(string, indent, "status",
                                             result_status_to_name(status));
@@ -622,10 +826,24 @@ append_test_result_to_string (GString *string, CutTestResult *result,
                                             start_time_string);
     g_free(start_time_string);
 
-    elapsed_string = g_strdup_printf("%f", cut_test_result_get_elapsed(result));
+    elapsed_string =
+        cut_utils_double_to_string(cut_test_result_get_elapsed(result));
     cut_utils_append_xml_element_with_value(string, indent, "elapsed",
                                             elapsed_string);
     g_free(elapsed_string);
+
+    if (expected)
+        cut_utils_append_xml_element_with_value(string, indent,
+                                                "expected", expected);
+    if (actual)
+        cut_utils_append_xml_element_with_value(string, indent,
+                                                "actual", actual);
+    if (diff)
+        cut_utils_append_xml_element_with_value(string, indent,
+                                                "diff", diff);
+    if (folded_diff)
+        cut_utils_append_xml_element_with_value(string, indent,
+                                                "folded-diff", folded_diff);
 }
 
 void
@@ -784,9 +1002,7 @@ cut_test_result_set_user_message (CutTestResult *result,
     if (user_message && user_message[0])
         priv->user_message = g_strdup(user_message);
 
-    if (priv->message)
-        g_free(priv->message);
-    priv->message = NULL;
+    reset_message(priv);
 }
 
 void
@@ -799,8 +1015,13 @@ cut_test_result_set_message (CutTestResult *result,
         g_free(priv->message);
         priv->message = NULL;
     }
-    if (message)
+
+    if (message) {
         priv->message = g_strdup(message);
+        priv->user_set_message = TRUE;
+    } else {
+        priv->user_set_message = FALSE;
+    }
 }
 
 void
@@ -816,9 +1037,7 @@ cut_test_result_set_system_message (CutTestResult *result,
     if (system_message && system_message[0])
         priv->system_message = g_strdup(system_message);
 
-    if (priv->message)
-        g_free(priv->message);
-    priv->message = NULL;
+    reset_message(priv);
 }
 
 void
@@ -849,6 +1068,106 @@ cut_test_result_set_elapsed (CutTestResult *result,
     CUT_TEST_RESULT_GET_PRIVATE(result)->elapsed = elapsed;
 }
 
+static void
+reset_diff (CutTestResultPrivate *priv)
+{
+    gboolean need_message_regeneration = FALSE;
+
+    if (!priv->user_set_diff) {
+        if (priv->diff)
+            g_free(priv->diff);
+        priv->diff = NULL;
+        need_message_regeneration = TRUE;
+    }
+
+    if (!priv->user_set_folded_diff) {
+        if (priv->folded_diff)
+            g_free(priv->folded_diff);
+        priv->folded_diff = NULL;
+        need_message_regeneration = TRUE;
+    }
+
+    if (need_message_regeneration)
+        reset_message(priv);
+}
+
+void
+cut_test_result_set_expected (CutTestResult *result, const gchar *expected)
+{
+    CutTestResultPrivate *priv;
+
+    priv = CUT_TEST_RESULT_GET_PRIVATE(result);
+
+    if (priv->expected) {
+        g_free(priv->expected);
+        priv->expected = NULL;
+    }
+
+    if (expected)
+        priv->expected = g_strdup(expected);
+
+    if (priv->expected)
+        reset_diff(priv);
+}
+
+void
+cut_test_result_set_actual (CutTestResult *result, const gchar *actual)
+{
+    CutTestResultPrivate *priv;
+
+    priv = CUT_TEST_RESULT_GET_PRIVATE(result);
+
+    if (priv->actual) {
+        g_free(priv->actual);
+        priv->actual = NULL;
+    }
+
+    if (actual)
+        priv->actual = g_strdup(actual);
+
+    if (priv->actual)
+        reset_diff(priv);
+}
+
+void
+cut_test_result_set_diff (CutTestResult *result, const gchar *diff)
+{
+    CutTestResultPrivate *priv;
+
+    priv = CUT_TEST_RESULT_GET_PRIVATE(result);
+
+    if (priv->diff) {
+        g_free(priv->diff);
+        priv->diff = NULL;
+    }
+
+    if (diff && diff[0]) {
+        priv->diff = g_strdup(diff);
+        priv->user_set_diff = TRUE;
+    } else {
+        priv->user_set_diff = FALSE;
+    }
+}
+
+void
+cut_test_result_set_folded_diff (CutTestResult *result, const gchar *folded_diff)
+{
+    CutTestResultPrivate *priv;
+
+    priv = CUT_TEST_RESULT_GET_PRIVATE(result);
+
+    if (priv->folded_diff) {
+        g_free(priv->folded_diff);
+        priv->folded_diff = NULL;
+    }
+
+    if (folded_diff && folded_diff[0]) {
+        priv->folded_diff = g_strdup(folded_diff);
+        priv->user_set_folded_diff = TRUE;
+    } else {
+        priv->user_set_folded_diff = FALSE;
+    }
+}
 
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
