@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2007-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2007-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -457,50 +457,42 @@ cut_utils_regex_replace (const gchar *pattern, const gchar *string,
 }
 
 const gchar *
-cut_utils_get_fixture_data_string (CutTestContext *context,
-                                   gchar **fixture_data_path,
-                                   const char *path,
-                                   ...)
+cut_utils_get_fixture_data (CutTestContext *context,
+                            const gchar **fixture_data_path,
+                            gsize *size,
+                            const char *path,
+                            ...)
 {
     const gchar *data;
     va_list args;
 
     va_start(args, path);
-    data = cut_utils_get_fixture_data_string_va_list(context, fixture_data_path,
-                                                     path, args);
+    data = cut_utils_get_fixture_data_va_list(context, fixture_data_path, size,
+                                              path, args);
     va_end(args);
 
     return data;
 }
 
 const gchar *
-cut_utils_get_fixture_data_string_va_list (CutTestContext *context,
-                                           gchar **fixture_data_path,
-                                           const gchar *path,
-                                           va_list args)
+cut_utils_get_fixture_data_va_list (CutTestContext *context,
+                                    const gchar **fixture_data_path,
+                                    gsize *size,
+                                    const gchar *path,
+                                    va_list args)
 {
-    GError *error = NULL;
-    const gchar *fixture_data;
+    GString *fixture_data;
 
-    fixture_data =
-        cut_test_context_get_fixture_data_string_va_list(context, &error,
-                                                         fixture_data_path,
-                                                         path, args);
-    if (error) {
-        gchar *inspected, *message;
+    fixture_data = gcut_utils_get_fixture_data_va_list(context,
+                                                       fixture_data_path,
+                                                       path, args);
+    if (!fixture_data)
+        return NULL;
 
-        inspected = gcut_error_inspect(error);
-        message = g_strdup_printf("can't get fixture data: %s", inspected);
-        g_error_free(error);
-        cut_test_context_register_result(context,
-                                         CUT_TEST_RESULT_ERROR,
-                                         message);
-        g_free(inspected);
-        g_free(message);
-        cut_test_context_long_jump(context);
-    }
+    if (size)
+        *size = fixture_data->len;
 
-    return fixture_data;
+    return fixture_data->str;
 }
 
 void
@@ -996,6 +988,19 @@ cut_win32_base_path (void)
     return win32_base_path;
 }
 
+static gchar *win32_icons_dir = NULL;
+
+const gchar *
+cut_win32_icons_dir (void)
+{
+    if (win32_icons_dir)
+        return win32_icons_dir;
+
+    win32_icons_dir = g_build_filename(cut_win32_base_path(), "share", PACKAGE,
+                                       "icons", NULL);
+    return win32_icons_dir;
+}
+
 gchar *
 cut_win32_build_module_dir_name (const gchar *type)
 {
