@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -34,6 +34,7 @@
 
 #include "gcut-io.h"
 #include "gcut-process.h"
+#include "gcut-glib-event-loop.h"
 #include "gcut-marshalers.h"
 #include "cut-utils.h"
 
@@ -81,6 +82,8 @@ struct _GCutProcessPrivate
     WatchOutputData *watch_error_data;
 
     guint forced_termination_wait_time;
+
+    GCutEventLoop *event_loop;
 };
 
 enum
@@ -152,9 +155,9 @@ gcut_process_class_init (GCutProcessClass *klass)
      * It is emitted each time an external process outputs
      * something to its standard output and it is read.
      *
-     * Note that you need to run GLib's main loop by
-     * g_main_loop_run(), g_main_context_iteration() and so
-     * on for detecting an external process's output is
+     * Note that you need to run %GCutEventLoop by
+     * gcut_event_loop_run() or gcut_event_loop_iterate()
+     * for detecting an external process's output is
      * readable.
      *
      * Since: 1.1.5
@@ -185,9 +188,9 @@ gcut_process_class_init (GCutProcessClass *klass)
      * something to its standard error output and it is
      * read.
      *
-     * Note that you need to run GLib's main loop by
-     * g_main_loop_run(), g_main_context_iteration() and so
-     * on for detecting an external process's output is
+     * Note that you need to run %GCutEventLoop by
+     * gcut_event_loop_run() or gcut_event_loop_iterate()
+     * for detecting an external process's output is
      * readable.
      *
      * Since: 1.1.5
@@ -214,9 +217,9 @@ gcut_process_class_init (GCutProcessClass *klass)
      *
      * It is emitted when an external process is exited.
      *
-     * Note that you need to run GLib's main loop by
-     * g_main_loop_run(), g_main_context_iteration() and so
-     * on for detecting an external process is exited.
+     * Note that you need to run %GCutEventLoop by
+     * gcut_event_loop_run() or gcut_event_loop_iterate()
+     * for detecting an external process is exited.
      *
      * Since: 1.1.5
      */
@@ -284,27 +287,35 @@ gcut_process_init (GCutProcess *process)
     priv->watch_error_data = NULL;
 
     priv->forced_termination_wait_time = 10;
+
+    priv->event_loop = NULL;
 }
 
 static void
 remove_child_watch_func (GCutProcessPrivate *priv)
 {
-    g_source_remove(priv->process_watch_id);
-    priv->process_watch_id = 0;
+    if (priv->process_watch_id > 0) {
+        gcut_event_loop_remove(priv->event_loop, priv->process_watch_id);
+        priv->process_watch_id = 0;
+    }
 }
 
 static void
 remove_output_watch_func (GCutProcessPrivate *priv)
 {
-    g_source_remove(priv->output_watch_id);
-    priv->output_watch_id = 0;
+    if (priv->output_watch_id > 0) {
+        gcut_event_loop_remove(priv->event_loop, priv->output_watch_id);
+        priv->output_watch_id = 0;
+    }
 }
 
 static void
 remove_error_watch_func (GCutProcessPrivate *priv)
 {
-    g_source_remove(priv->error_watch_id);
-    priv->error_watch_id = 0;
+    if (priv->error_watch_id > 0) {
+        gcut_event_loop_remove(priv->event_loop, priv->error_watch_id);
+        priv->error_watch_id = 0;
+    }
 }
 
 static gboolean
@@ -323,11 +334,8 @@ dispose_io_channel (GCutProcessPrivate *priv)
         priv->input = NULL;
     }
 
-    if (priv->output_watch_id > 0)
-        remove_output_watch_func(priv);
-
-    if (priv->error_watch_id > 0)
-        remove_error_watch_func(priv);
+    remove_output_watch_func(priv);
+    remove_error_watch_func(priv);
 
     if (priv->output) {
         g_io_channel_unref(priv->output);
@@ -377,6 +385,35 @@ gcut_process_close (GCutProcess *process)
     }
 }
 
+static void
+dispose_process_watch_id (GCutProcess *process)
+{
+    GCutProcessPrivate *priv;
+    GError *error = NULL;
+    gboolean is_timeout = FALSE;
+    guint timeout_wait_id;
+
+    priv = GCUT_PROCESS_GET_PRIVATE(process);
+
+    if (priv->process_watch_id == 0 || priv->pid == 0)
+        return;
+
+    gcut_process_kill(process, SIGTERM, &error);
+    if (priv->forced_termination_wait_time == 0)
+        return;
+
+    timeout_wait_id =
+        gcut_event_loop_add_timeout_full(priv->event_loop,
+                                         G_PRIORITY_LOW,
+                                         priv->forced_termination_wait_time,
+                                         cb_timeout_wait,
+                                         &is_timeout,
+                                         NULL);
+    while (!is_timeout && priv->process_watch_id > 0 && priv->pid > 0)
+        gcut_event_loop_iterate(priv->event_loop, TRUE);
+    g_source_remove(timeout_wait_id);
+}
+
 static void
 dispose (GObject *object)
 {
@@ -388,24 +425,9 @@ dispose (GObject *object)
         priv->command = NULL;
     }
 
-    if (priv->process_watch_id && priv->pid) {
-        GError *error = NULL;
-        gcut_process_kill(GCUT_PROCESS(object), SIGTERM, &error);
-        if (priv->forced_termination_wait_time > 0) {
-            gboolean is_timeout = FALSE;
-            guint timeout_wait_id;
+    dispose_process_watch_id(GCUT_PROCESS(object));
 
-            timeout_wait_id = g_timeout_add(priv->forced_termination_wait_time,
-                                            cb_timeout_wait,
-                                            &is_timeout);
-            while (!is_timeout && priv->process_watch_id > 0 && priv->pid > 0)
-                g_main_context_iteration(NULL, TRUE);
-            g_source_remove(timeout_wait_id);
-        }
-    }
-
-    if (priv->process_watch_id)
-        remove_child_watch_func(priv);
+    remove_child_watch_func(priv);
 
     if (priv->pid) {
         gcut_process_close(GCUT_PROCESS(object));
@@ -424,6 +446,11 @@ dispose (GObject *object)
 #endif
     dispose_io_channel(priv);
 
+    if (priv->event_loop) {
+        g_object_unref(priv->event_loop);
+        priv->event_loop = NULL;
+    }
+
     G_OBJECT_CLASS(gcut_process_parent_class)->dispose(object);
 }
 
@@ -655,9 +682,7 @@ static void
 child_watch_func (GPid pid, gint status, gpointer data)
 {
     GCutProcess *process = data;
-    GCutProcessPrivate *priv;
 
-    priv = GCUT_PROCESS_GET_PRIVATE(process);
     reap_child(process, pid, status);
 }
 
@@ -686,7 +711,7 @@ create_output_channel (gint fd)
 }
 
 static GIOChannel *
-create_input_channel (gint fd, guint *watch_id,
+create_input_channel (GCutEventLoop *loop, gint fd, guint *watch_id,
                       GIOFunc watch_func, gpointer user_data)
 {
     GIOChannel *channel;
@@ -697,10 +722,11 @@ create_input_channel (gint fd, guint *watch_id,
 
     g_io_channel_set_flags(channel, G_IO_FLAG_NONBLOCK, NULL);
 
-    *watch_id = g_io_add_watch(channel,
-                               G_IO_IN | G_IO_PRI |
-                               G_IO_ERR | G_IO_HUP | G_IO_NVAL,
-                               watch_func, user_data);
+    *watch_id = gcut_event_loop_watch_io(loop,
+                                         channel,
+                                         G_IO_IN | G_IO_PRI |
+                                         G_IO_ERR | G_IO_HUP | G_IO_NVAL,
+                                         watch_func, user_data);
 
     return channel;
 }
@@ -745,10 +771,8 @@ error_received (GCutProcess *process,
 static gboolean
 read_from_io_channel (GIOChannel *channel, GCutProcess *process, guint signal)
 {
-    GCutProcessPrivate *priv;
     gboolean need_more_data = TRUE;
 
-    priv = GCUT_PROCESS_GET_PRIVATE(process);
     while (need_more_data) {
         GIOStatus status;
         gchar stream[BUFFER_SIZE];
@@ -813,6 +837,7 @@ gcut_process_run (GCutProcess *process, GError **error)
     gboolean success;
     gint input_fd = 0, output_fd = 0, error_fd = 0;
     gchar **env;
+    GCutEventLoop *loop;
 
     priv = GCUT_PROCESS_GET_PRIVATE(process);
 
@@ -854,9 +879,14 @@ gcut_process_run (GCutProcess *process, GError **error)
     if (!success)
         return FALSE;
 
-    priv->process_watch_id = g_child_watch_add(priv->pid,
-                                               child_watch_func,
-                                               process);
+    loop = gcut_process_get_event_loop(process);
+    priv->process_watch_id =
+        gcut_event_loop_watch_child_full(loop,
+                                         G_PRIORITY_LOW,
+                                         priv->pid,
+                                         child_watch_func,
+                                         process,
+                                         NULL);
 
     if (input_fd > 0)
         priv->input = create_output_channel(input_fd);
@@ -865,7 +895,8 @@ gcut_process_run (GCutProcess *process, GError **error)
         priv->watch_output_data = g_new(WatchOutputData, 1);
         priv->watch_output_data->process = process;
         priv->watch_output_data->signal = signals[OUTPUT_RECEIVED];
-        priv->output = create_input_channel(output_fd,
+        priv->output = create_input_channel(loop,
+                                            output_fd,
                                             &(priv->output_watch_id),
                                             watch_output,
                                             priv->watch_output_data);
@@ -874,7 +905,8 @@ gcut_process_run (GCutProcess *process, GError **error)
         priv->watch_error_data = g_new(WatchOutputData, 1);
         priv->watch_error_data->process = process;
         priv->watch_error_data->signal = signals[ERROR_RECEIVED];
-        priv->error = create_input_channel(error_fd,
+        priv->error = create_input_channel(loop,
+                                           error_fd,
                                            &(priv->error_watch_id),
                                            watch_output,
                                            priv->watch_error_data);
@@ -943,6 +975,7 @@ gint
 gcut_process_wait (GCutProcess *process, guint timeout, GError **error)
 {
     GCutProcessPrivate *priv;
+    GCutEventLoop *loop;
     gboolean is_timeout = FALSE;
     guint timeout_id;
 
@@ -976,10 +1009,16 @@ gcut_process_wait (GCutProcess *process, guint timeout, GError **error)
         return -1;
     }
 
-    timeout_id = g_timeout_add(timeout, cb_timeout_wait, &is_timeout);
+    loop = gcut_process_get_event_loop(process);
+    timeout_id = gcut_event_loop_add_timeout_full(loop,
+                                                  G_PRIORITY_LOW,
+                                                  timeout,
+                                                  cb_timeout_wait,
+                                                  &is_timeout,
+                                                  NULL);
     while (!is_timeout && priv->pid > 0)
-        g_main_context_iteration(NULL, TRUE);
-    g_source_remove(timeout_id);
+        gcut_event_loop_iterate(priv->event_loop, TRUE);
+    gcut_event_loop_remove(loop, timeout_id);
 
     if (is_timeout) {
         gchar *command;
@@ -1122,6 +1161,39 @@ gcut_process_get_error_stream (GCutProcess *process)
 }
 #endif
 
+GCutEventLoop *
+gcut_process_get_event_loop (GCutProcess *process)
+{
+    GCutProcessPrivate *priv;
+
+    g_return_val_if_fail(GCUT_IS_PROCESS(process), NULL);
+
+    priv = GCUT_PROCESS_GET_PRIVATE(process);
+
+    if (!priv->event_loop) {
+        priv->event_loop = gcut_glib_event_loop_new(NULL);
+    }
+
+    return priv->event_loop;
+}
+
+void
+gcut_process_set_event_loop (GCutProcess *process, GCutEventLoop *loop)
+{
+    GCutProcessPrivate *priv;
+
+    g_return_if_fail(GCUT_IS_PROCESS(process));
+
+    priv = GCUT_PROCESS_GET_PRIVATE(process);
+    if (priv->event_loop == loop)
+        return;
+
+    g_object_ref(loop);
+    if (priv->event_loop)
+        g_object_unref(priv->event_loop);
+    priv->event_loop = loop;
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
