@@ -4,15 +4,28 @@ require 'time'
 
 if ARGV.size < 4
   puts "Usage: #{$0} " +
+         "PACKAGE " +
          "OLD_VERSION OLD_RELEASE_DATE " +
          "NEW_VERSION NEW_RELEASE_DATE " +
          "FILES..."
-  puts " e.g.: #{$0} 1.0.9 2009-10-20 1.1.0 2009-11-29 " +
+  puts " e.g.: #{$0} cutter 1.0.9 2009-10-20 1.1.0 2009-11-29 " +
          " html/index.html html/index.html.ja rpm/fedora/cutter.spec.in"
   exit(1)
 end
 
-old_version, old_release_date, new_version, new_release_date, *files = ARGV
+package, old_version, old_release_date, new_version, new_release_date, *files =
+  ARGV
+
+def git_user_name
+  `git config --get user.name`.chomp
+end
+
+def git_user_email
+  `git config --get user.email`.chomp
+end
+
+name = ENV['DEBFULLNAME'] || ENV['NAME'] || git_user_name
+email = ENV['DEBEMAIL'] || ENV['EMAIL'] || git_user_email
 
 files.each do |file|
   content = replaced_content = File.read(file)
@@ -22,9 +35,20 @@ files.each do |file|
     if content !~ /#{Regexp.escape(new_version)}/
       replaced_content = content.sub(/^(%changelog\n)/, <<-EOC)
 %changelog
-* #{date} Kouhei Sutou <kou@clear-code.com>
-- (#{new_version}-0)
-- new upstream release
+* #{date} #{name} <#{email}> - #{new_version}-0
+- new upstream release.
+
+      EOC
+    end
+  when /debian\/changelog\z/
+    date = Time.parse(new_release_date).rfc2822
+    if content !~ /#{Regexp.escape(new_version)}/
+      replaced_content = content.sub(/\A/, <<-EOC)
+#{package} (#{new_version}-1) unstable; urgency=low
+
+  * New upstream release.
+
+ -- #{name} <#{email}>  #{date}
 
       EOC
     end
