@@ -1,26 +1,25 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
  *  Copyright (C) 2008  g新部 Hiroyuki Ikezoe  <poincare@ikezoe.net>
+ *  Copyright (C) 2009-2011  Kouhei Sutou  <kou@clear-code.com>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2 of the License, or (at your option) any later version.
+ *  This library is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
  *
  *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this program; if not, write to the
- *  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- *  Boston, MA  02111-1307  USA
+ *  You should have received a copy of the GNU Lesser General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
 #ifdef HAVE_CONFIG_H
-#  include <cutter/config.h>
+#  include <config.h>
 #endif /* HAVE_CONFIG_H */
 
 #include <stdlib.h>
@@ -44,7 +43,6 @@
 #  include <goffice/graph/gog-label.h>
 #  include <goffice/graph/gog-series.h>
 #  include <goffice/graph/gog-data-set.h>
-#  include <goffice/graph/gog-style.h>
 #  include <goffice/graph/gog-styled-object.h>
 #endif
 
@@ -113,6 +111,8 @@ static gchar   *get_error_results        (CutReport   *report);
 static gchar   *get_failure_results      (CutReport   *report);
 static gchar   *get_pending_results      (CutReport   *report);
 static gchar   *get_notification_results (CutReport   *report);
+static gchar   *get_omission_results     (CutReport   *report);
+static gchar   *get_crash_results        (CutReport   *report);
 static gchar   *get_test_result          (CutReport   *report,
                                           const gchar *test_name);
 
@@ -138,6 +138,8 @@ class_init (CutPDFReportClass *klass)
     report_class->get_failure_results      = get_failure_results;
     report_class->get_pending_results      = get_pending_results;
     report_class->get_notification_results = get_notification_results;
+    report_class->get_omission_results     = get_omission_results;
+    report_class->get_crash_results        = get_crash_results;
     report_class->get_test_result          = get_test_result;
 
     spec = g_param_spec_object("cut-run-context",
@@ -414,7 +416,7 @@ show_summary (CutPDFReport *report, CutRunContext *run_context)
     const GList *node;
     CutTestCase *test_case = NULL;
     cairo_t *cr;
-    gdouble x, y, after_x, after_y, width, height;
+    gdouble x, y, after_x, after_y;
     guint n_tests, n_successes, n_failures, n_errors, n_pendings, n_omissions;
     CutTestResultStatus test_case_status;
 
@@ -472,8 +474,6 @@ show_summary (CutPDFReport *report, CutRunContext *run_context)
                        n_errors, n_pendings, n_omissions);
 
     cairo_get_current_point(cr, &after_x, &after_y);
-    width = A4_WIDTH - x * 2;
-    height = after_y - y;
 }
 
 #ifdef HAVE_GOFFICE
@@ -482,17 +482,17 @@ graph_setup_title (GogGraph *graph, const gchar *title)
 {
     GogLabel *label;
     GOData *data;
-    GogStyle *style;
+    GOStyle *style;
     PangoFontDescription *desc;
 
-    label = g_object_new(GOG_LABEL_TYPE, NULL);
+    label = g_object_new(GOG_TYPE_LABEL, NULL);
     data = go_data_scalar_str_new(title, FALSE);
     gog_dataset_set_dim(GOG_DATASET(label), 0, data, NULL);
     gog_object_add_by_name(GOG_OBJECT(graph), "Title", GOG_OBJECT(label));
 
-    style = gog_styled_object_get_style(GOG_STYLED_OBJECT(label));
+    style = go_styled_object_get_style(GO_STYLED_OBJECT(label));
     desc = pango_font_description_from_string("Sans bold 16");
-    gog_style_set_font_desc(style, desc);
+    go_style_set_font_desc(style, desc);
 }
 
 static void
@@ -502,20 +502,26 @@ graph_setup_chart (GogGraph *graph, CutRunContext *run_context)
     GogPlot *pie;
     GOData *data;
     GogSeries *series;
-    const gchar *legends[CUT_TEST_RESULT_LAST];
-    double values[CUT_TEST_RESULT_LAST];
+    gchar **legends;
+    double *values;
+    gint n = 0;
 
     chart = GOG_CHART(gog_object_get_child_by_name(GOG_OBJECT(graph), "Chart"));
     pie = gog_plot_new_by_name("GogPiePlot");
     gog_object_add_by_name(GOG_OBJECT(chart), "Plot", GOG_OBJECT(pie));
 
+    legends = g_new0(gchar *, CUT_TEST_RESULT_LAST);
+    values = g_new0(double, CUT_TEST_RESULT_LAST);
+
 #define STATUS(status) (CUT_TEST_RESULT_ ## status)
 #define SET_DATA(status, n_statuses) G_STMT_START                       \
 {                                                                       \
-    legends[STATUS(status)] =                                           \
-        cut_test_result_status_to_signal_name(STATUS(status));          \
+    const gchar *name;                                                  \
+    name = cut_test_result_status_to_signal_name(STATUS(status));       \
+    legends[STATUS(status)] = g_strdup(name);                           \
     values[STATUS(status)] =                                            \
         cut_run_context_get_n_ ## n_statuses(run_context);              \
+    n++;                                                                \
 } G_STMT_END                                                            \
 
     SET_DATA(SUCCESS, successes);
@@ -529,9 +535,10 @@ graph_setup_chart (GogGraph *graph, CutRunContext *run_context)
 #undef SET_DATA
 
     series = gog_plot_new_series(pie);
-    data = go_data_vector_str_new(legends, CUT_TEST_RESULT_LAST, NULL);
+    data = go_data_vector_str_new((const gchar * const *)legends, n,
+                                  (GDestroyNotify)g_strfreev);
     gog_series_set_dim(series, 0, data, NULL);
-    data = go_data_vector_val_new(values, CUT_TEST_RESULT_LAST, NULL);
+    data = go_data_vector_val_new(values, n, g_free);
     gog_series_set_dim(series, 1, data, NULL);
     gog_object_add_by_name(GOG_OBJECT(chart), "Legend", NULL);
 }
@@ -541,7 +548,7 @@ graph_new (const gchar *title, CutRunContext *run_context)
 {
     GogGraph *graph;
 
-    graph = g_object_new(GOG_GRAPH_TYPE, NULL);
+    graph = g_object_new(GOG_TYPE_GRAPH, NULL);
     gog_object_add_by_name(GOG_OBJECT(graph), "Chart", NULL);
 
     graph_setup_title(graph, title);
@@ -577,12 +584,6 @@ cb_complete_run (CutRunContext *run_context, gboolean success,
     show_summary(report, run_context);
 }
 
-static void
-cb_crashed (CutRunContext *run_context, const gchar *backtrace,
-            CutPDFReport *report)
-{
-}
-
 static void
 connect_to_run_context (CutPDFReport *report, CutRunContext *run_context)
 {
@@ -602,14 +603,14 @@ connect_to_run_context (CutPDFReport *report, CutRunContext *run_context)
     CONNECT_TO_TEST(error_test);
     CONNECT_TO_TEST(pending_test);
     CONNECT_TO_TEST(notification_test);
+    CONNECT_TO_TEST(omission_test);
+    CONNECT_TO_TEST(crash_test);
 
     CONNECT(complete_test);
     CONNECT(complete_test_case);
     CONNECT(complete_test_suite);
     CONNECT(complete_run);
 
-    CONNECT(crashed);
-
 #undef CONNECT
 }
 
@@ -631,8 +632,6 @@ disconnect_from_run_context (CutPDFReport *report, CutRunContext *run_context)
     DISCONNECT(complete_test_suite);
     DISCONNECT(complete_run);
 
-    DISCONNECT(crashed);
-
     g_signal_handlers_disconnect_by_func(run_context,
                                          G_CALLBACK(cb_test_signal),
                                          report);
@@ -704,6 +703,18 @@ get_notification_results (CutReport *report)
     return NULL;
 }
 
+static gchar *
+get_omission_results (CutReport *report)
+{
+    return NULL;
+}
+
+static gchar *
+get_crash_results (CutReport *report)
+{
+    return NULL;
+}
+
 static gchar *
 get_test_result (CutReport *report, const gchar *test_name)
 {
